note
	description: "[
		Elements that contain an ordered list of zero or more children elements.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_COMPOUND_ELEMENT [G -> EDL_ELEMENT]

inherit
	EDL_ELEMENT
		redefine
			default_create,
			has
		end

feature {NONE} -- Initialization

	default_create
			-- Initialize with no children elements.
		do
			create {LINKED_LIST [G]} children.make
		ensure then
			no_childern: count = 0
		end

	make_from_element (a_element: G)
			-- Initialize with one child `a_element'.
		require
			element_has_no_parent: a_element.parent = Void
			valid_child: is_valid_child (a_element)
		do
			default_create
			extend (a_element)
		ensure
			correct_count: count = 1
			parent_set: a_element.parent = Current
			first_child_correct: first = a_element
			last_child_correct: last = a_element
		end

	make_from_elements (a_elements: ARRAY [G])
			-- Initialize with children elements in `a_elements'.
		require
			elements_have_no_parent: a_elements.for_all (agent (a_element: G): BOOLEAN
				do
					Result := a_element.parent = Void
				end)
			valid_children: are_valid_children (a_elements)
		do
			default_create
			extend_with_elements (a_elements)
		ensure
			correct_count: count = a_elements.count
			parent_set: a_elements.for_all (agent (a_element: G): BOOLEAN
				do
					Result := a_element.parent = Current
				end)
		end

feature -- Access

	has (a_element: EDL_ELEMENT): BOOLEAN
			-- Is `a_element' equivalent to the compound element or one of its heir
			-- elements (by reference)?
		local
			i: INTEGER
		do
			Result := Current = a_element
			
			from
				i := children.index
				children.start
			until
				children.after or Result
			loop
				Result := children.item.has (a_element)
				children.forth
			end

			children.go_i_th (i)
		ensure then
			correct: Result implies (count > 0 or Current = a_element)
		end

	first: G
			-- The first child element.
		require
			non_empty: count > 0
		do
			Result := children.first
		end

	last: G
			-- The last child element.
		require
			non_empty: count > 0
		do
			Result := children.last
		end

feature -- Measurement

	count: INTEGER
			-- Number of children elements.
		do
			-- `count' is used in preconditions of creation procedures provided by some
			-- subclasses of `EDL_COMPOUND_ELEMENT'. At the time of executing these
			-- preconditions, `children' is not attached yet.
			if attached children then
				Result := children.count
			else
				Result := 0
			end
		ensure
			non_negative_count: Result >= 0
		end

feature -- Element change

	extend (a_element: G)
			-- Add `a_element' to the end of the list of children elements.
		require
			element_has_no_parent: a_element.parent = Void
			no_cycle: not a_element.has (Current)
			valid_child: is_valid_child (a_element)
		do
			children.extend (a_element)
			a_element.parent := Current
		ensure
			correct_count: count = old count + 1
			parent_set: a_element.parent = Current
			last_child_correct: last = a_element
		end

	extend_with_elements (a_elements: ARRAY [G])
			-- Add all elements in `a_elements'.
		require
			elements_have_no_parent: a_elements.for_all (agent (a_element: G): BOOLEAN
				do
					Result := a_element.parent = Void
				end)
			no_cycle: not a_elements.there_exists (agent (a_element: G): BOOLEAN
				do
					Result := a_element.has (Current)
				end)
			valid_children: are_valid_children (a_elements)
		do
			a_elements.do_all (agent extend)
		ensure
			correct_count: count = old count + a_elements.count
			parent_set: a_elements.for_all (agent (a_element: G): BOOLEAN
				do
					Result := a_element.parent = Current
				end)
		end

feature -- Miscellaneous

	is_valid_child (a_element: G): BOOLEAN
			-- Is it ok to add `a_element'? (Default answer: Yes)
		do
			Result := True
		end

	are_valid_children (a_elements: ARRAY [G]): BOOLEAN
			-- Is it ok to add `a_elements'?
		do
			Result := a_elements.for_all (agent is_valid_child)
		ensure
			valid_implies_valid_children: Result implies a_elements.for_all (agent is_valid_child)
		end

	are_strings_utf8 (a_strings: INDEXABLE [STRING, INTEGER]): BOOLEAN
			-- Are all strings in `a_strings' valid UTF-8-encoded?
		local
			utf8_encoding: EDL_UTF8_ENCODING
			i: INTEGER
		do
			Result := True

			check attached a_strings.index_set as interval then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				from
					create utf8_encoding
					i := interval.lower
				until
					i > interval.upper or not Result
				loop
					Result := utf8_encoding.is_valid (a_strings.item (i))
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

	children: LIST [G]
			-- List of all children elements.

	accept_children (a_visitor: EDL_VISITOR)
			-- Pass `a_visitor' to all children elements.
		do
			from
				children.start
			until
				children.after
			loop
				children.item.accept (a_visitor)
				children.forth
			end
		end

	extend_with_strings (a_strings: INDEXABLE [STRING, INTEGER]; a_transformation: FUNCTION [ANY, TUPLE [STRING], G])
			-- Append elements created by `a_transformation' from `a_strings' to `children'.
		require
			strings_are_utf8: are_strings_utf8 (a_strings)
		local
			i: INTEGER
		do
			check attached a_strings.index_set as interval then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				from
					i := interval.lower
				until
					i > interval.upper
				loop
					extend (a_transformation.item ([a_strings.item (i)]))
					i := i + 1
				end
			end
		ensure
			correct_count: attached a_strings.index_set as index_set implies count = old count + index_set.count
		end

invariant

	non_negative_count: count >= 0

end
