note
	description: "[
		Elements used to include images in a document.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_IMAGE

inherit
	EDL_EXTERNAL_ELEMENT
		redefine
			supports_file
		end

	EDL_INLINE_ELEMENT

	EDL_AUTONOMOUS_ELEMENT

create
	make

feature -- Miscellaneous

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor'.
		do
			a_visitor.visit_image (Current)
		end

	supports_file (a_file: FILE): BOOLEAN
			-- Does `a_file' have a supported file extension and magic number?
		local
			old_index: INTEGER
			old_closed: BOOLEAN
			has_matched: BOOLEAN
			extension: STRING
		do
			Result := Precursor (a_file)
			extension := (create {EDL_FILE_NAME}.make_from_file (a_file)).extension

			check attached extension.as_lower as lower_case_file_extension then
					-- Is always true since `as_lower' has a post-condition that
					-- prevents it from being Void.
				extension := lower_case_file_extension
			end

			if Result then
				old_index := a_file.index

				if a_file.is_closed then
					old_closed := True
					a_file.open_read
				end

				from
					supported_magic_numbers.start
				until
					supported_magic_numbers.after or has_matched
				loop
					a_file.go (0)
					a_file.read_stream (supported_magic_numbers.key_for_iteration.count)

					if supported_magic_numbers.key_for_iteration.is_equal (a_file.last_string) then
						has_matched := supported_magic_numbers.item_for_iteration.has (extension)
					end

					supported_magic_numbers.forth
				end

				if not has_matched then
					Result := False
				end

				a_file.go (old_index)

				if old_closed then
					a_file.close
				end
			end
		end

	supports_file_extension (a_file_extension: STRING): BOOLEAN
			-- Is `a_file_extension' a supported image file extension?
		do
			check attached a_file_extension.as_lower as lower_case_file_extension then
					-- Is always true since `as_lower' has a post-condition that
					-- prevents it from being Void.
				Result := supported_file_extensions.has (lower_case_file_extension)
			end
		end

	supported_file_extensions: LINKED_SET [STRING]
			-- List of supported file extensions in lower-case.
		once
			create Result.make

			Result.compare_objects

			from
				supported_magic_numbers.start
			until
				supported_magic_numbers.after
			loop
				Result.append (supported_magic_numbers.item_for_iteration)
				supported_magic_numbers.forth
			end
		end

	supported_magic_numbers: HASH_TABLE [LIST [STRING], STRING]
			-- Mapping of magic numbers to lists of corresponding file extensions.
		local
			magic_number: STRING
			extensions: LINKED_LIST [STRING]
		once
			create Result.make (4)

			-- PNG
			magic_number := ""
			magic_number.append_code (0x89)
			magic_number.append_code (0x50)
			magic_number.append_code (0x4E)
			magic_number.append_code (0x47)
			magic_number.append_code (0x0D)
			magic_number.append_code (0x0A)
			magic_number.append_code (0x1A)
			magic_number.append_code (0x0A)
			create extensions.make
			extensions.compare_objects
			extensions.extend ("png")
			Result.put (extensions, magic_number)

			-- JPEG
			magic_number := ""
			magic_number.append_code (0xFF)
			magic_number.append_code (0xD8)
			create extensions.make
			extensions.compare_objects
			extensions.extend ("jpg")
			extensions.extend ("jpeg")
			Result.put (extensions, magic_number)

			-- GIF87a
			magic_number := ""
			magic_number.append_code (0x47)
			magic_number.append_code (0x49)
			magic_number.append_code (0x46)
			magic_number.append_code (0x38)
			magic_number.append_code (0x37)
			magic_number.append_code (0x61)
			create extensions.make
			extensions.compare_objects
			extensions.extend ("gif")
			Result.put (extensions, magic_number)

			-- GIF89a
			magic_number := ""
			magic_number.append_code (0x47)
			magic_number.append_code (0x49)
			magic_number.append_code (0x46)
			magic_number.append_code (0x38)
			magic_number.append_code (0x39)
			magic_number.append_code (0x61)
			create extensions.make
			extensions.compare_objects
			extensions.extend ("gif")
			Result.put (extensions, magic_number)
		end

end
