note
	description: "[
		References can be used to refer to one of three things:
		 - URI
		 - section of a document inside the same document collection
		 - another document of the same document collection
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_REFERENCE

inherit
	EDL_INLINE_ELEMENT
		undefine
			default_create,
			has
		end

	EDL_COMPOUND_ELEMENT [EDL_INLINE_ELEMENT]
		redefine
			is_valid_child
		end

create
	make_from_uri,
	make_from_section,
	make_from_document,
	make_simple_from_uri,
	make_simple_from_section,
	make_simple_from_document

feature {NONE} -- Initialization

	make_from_uri (a_uri: STRING)
			-- Initialize with `a_uri' as destination.
		require
			valid_uri: is_valid_uri (a_uri)
		do
			default_create
			destination_uri := a_uri
		ensure
			uri_set: destination_uri = a_uri
			destination_is_not_section: not attached destination_section
			destination_is_not_document: not attached destination_document
			no_children: count = 0
		end

	make_from_section (a_section: EDL_SECTION)
			-- Initialize with `a_section' as destination.
		do
			default_create
			destination_section := a_section
		ensure
			section_set: destination_section = a_section
			destination_is_not_uri: not attached destination_uri
			destination_is_not_document: not attached destination_document
			no_children: count = 0
		end

	make_from_document (a_document: EDL_DOCUMENT)
			-- Initialize with `a_document' as destination.
		do
			default_create
			destination_document := a_document
		ensure
			document_set: destination_document = a_document
			destination_is_not_section: not attached destination_section
			destination_is_not_uri: not attached destination_uri
			no_children: count = 0
		end

	make_simple_from_uri (a_uri: STRING; a_string: STRING)
			-- Initialize with `a_uri' as destination and content `a_string'.
		require
			valid_uri: is_valid_uri (a_uri)
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			make_from_uri (a_uri)
			extend_simple (a_string)
		ensure
			uri_set: destination_uri = a_uri
			destination_is_not_section: not attached destination_section
			destination_is_not_document: not attached destination_document
			correct_count: count = 1
		end

	make_simple_from_section (a_section: EDL_SECTION; a_string: STRING)
			-- Initialize with `a_section' as destination and content `a_string'.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			make_from_section (a_section)
			extend_simple (a_string)
		ensure
			section_set: destination_section = a_section
			destination_is_not_section: not attached destination_uri
			destination_is_not_document: not attached destination_document
			correct_count: count = 1
		end

	make_simple_from_document (a_document: EDL_DOCUMENT; a_string: STRING)
			-- Initialize with `a_document' as destination and content `a_string'.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			make_from_document (a_document)
			extend_simple (a_string)
		ensure
			document_set: destination_document = a_document
			destination_is_not_section: not attached destination_section
			destination_is_not_uri: not attached destination_uri
			correct_count: count = 1
		end

feature -- Access

	destination_uri: detachable STRING
			-- Destination URI if `destination_is_uri'.

	destination_section: detachable EDL_SECTION
			-- Destination section if `destination_is_section'.

	destination_document: detachable EDL_DOCUMENT
			-- Destination document if `destination_is_document'.

feature -- Element change

	extend_simple (a_string: STRING)
			-- Add a text element with `a_string' as content.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			extend (create {EDL_TEXT_ELEMENT}.make (a_string))
		ensure
			correct_count: count = old count + 1
		end

feature -- Miscellaneous

	is_valid_child (a_child: EDL_INLINE_ELEMENT): BOOLEAN
			-- Is it ok to add `a_child'? Answer: Only if `a_child' isn't
			-- a reference.
		do
			Result := not attached {EDL_REFERENCE} a_child
		end

	is_valid_uri (a_uri: STRING): BOOLEAN
			-- Is `a_uri' valid according to RFC 2396?
		do
			uri_regex.match (a_uri)

			Result := uri_regex.has_matched
		end

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor' and pass it on to the children.
		do
			a_visitor.visit_reference (Current)
			accept_children (a_visitor)
			a_visitor.leave_reference (Current)
		end

feature {NONE} -- Implementation

	uri_regex: RX_PCRE_REGULAR_EXPRESSION
			-- Compiled regular expression to check URI validity according to RFC 2396.
		local
			s: STRING
		once
			create s.make_empty

			s.append ("(?:%(A-Za-z%)%(%Hx2b%Hx2d%Hx2e0-9A-Za-z%)*:")
			s.append ("(?:(?://")
			s.append ("(?:(?:(?:(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-;=A-Z_a-z%T%))*%A)?")
			s.append ("(?:(?:(?:%(0-9A-Za-z%)%V%(0-9A-Za-z%)%(%Hx2d0-9A-Za-z%)*%(0-9A-Za-z%))")
			s.append ("%Hx2e)*(?:%(A-Za-z%)%V%(A-Za-z%)%(%Hx2d0-9A-Za-z%)*%(0-9A-Za-z%))")
			s.append ("%Hx2e?%V")
			s.append ("%Hd+%Hx2e%Hd+%Hx2e%Hd+%Hx2e%Hd+)(?::%Hd*)?)?%V")
			s.append ("(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-;=%A-Z_a-z%T%))+)")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;")
			s.append ("(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V")
			s.append ("%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*)*)?%V")
			s.append ("/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*)*)")
			s.append ("(?:%Hx3f(?:%(!%Hx24&-;=%Hx3f-Z_a-z%T%)%V%%%(0-9A-Fa-f%)%(0-9A-Fa-f%))*)?%V")
			s.append ("(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-;=%Hx3f-Z_a-z%T%))")
			s.append ("(?:%(!%Hx24&-;=%Hx3f-Z_a-z%T%)%V%%%(0-9A-Fa-f%)%(0-9A-Fa-f%))*)%V")
			s.append ("(?://")
			s.append ("(?:(?:(?:(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-;=A-Z_a-z%T%))*%A)?")
			s.append ("(?:(?:(?:%(0-9A-Za-z%)%V%(0-9A-Za-z%)%(%Hx2d0-9A-Za-z%)*%(0-9A-Za-z%))")
			s.append ("%Hx2e)*(?:%(A-Za-z%)%V%(A-Za-z%)%(%Hx2d0-9A-Za-z%)*%(0-9A-Za-z%))")
			s.append ("%Hx2e?%V")
			s.append ("%Hd+%Hx2e%Hd+%Hx2e%Hd+%Hx2e%Hd+)(?::%Hd*)?)?%V")
			s.append ("(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-;=%A-Z_a-z%T%))+)")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;")
			s.append ("(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*)*)?%V")
			s.append ("/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*)*%V")
			s.append ("(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-9;=%A-Z_a-z%T%))+")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*")
			s.append ("(?:/(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*")
			s.append ("(?:;")
			s.append ("(?:%%%(0-9A-Fa-f%)%(0-9A-Fa-f%)%V%(!%Hx24&-%Hx2e0-:=%A-Z_a-z%T%))*)*)*)?)")
			s.append ("(?:%Hx3f(?:%(!%Hx24&-;=%Hx3f-Z_a-z%T%)%V%%%(0-9A-Fa-f%)%(0-9A-Fa-f%))*)?)?")
			s.append ("(?:%Hx23(?:%(!%Hx24&-;=%Hx3f-Z_a-z%T%)%V%%%(0-9A-Fa-f%)%(0-9A-Fa-f%))*)?")

			create Result.make

			Result.set_extended (True)
			Result.set_anchored (True)
			Result.set_dollar_endonly (True)
			Result.set_empty_allowed (False)
			Result.set_strict (True)
			Result.compile (s)
		end

invariant

	at_most_one_destination_uri: attached destination_uri implies (not attached destination_section and not attached destination_document)
	at_most_one_destination_section: attached destination_section implies (not attached destination_uri and not attached destination_document)
	at_most_one_destination_document: attached destination_document implies (not attached destination_uri and not attached destination_section)
	at_least_one_destination: attached destination_uri or attached destination_section or attached destination_document
	valid_uri: attached destination_uri as uri implies is_valid_uri (uri)

end
