note
	description: "[
		Constraints on the number of children of elements of type `G' in a document.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_CHILDREN_COUNT_CONSTRAINT [G -> EDL_COMPOUND_ELEMENT [EDL_ELEMENT]]

inherit
	EDL_DOCUMENT_CONSTRAINT

	EDL_SINK_VISITOR
		redefine
			visit_document
		end

create
	make_minimum,
	make_maximum,
	make_range

feature {NONE} -- Initialization

	make_minimum (a_min_count: INTEGER)
			-- Initialize a constraint that only allows elements that conform
			-- to type `G' if they have at least `a_min_count' children.
		require
			positive_min_count: a_min_count > 0
		do
			min_count := a_min_count
			has_lower_bound := True
		ensure
			lower_bound_set: has_lower_bound
			min_count_set: min_count = a_min_count
		end

	make_maximum (a_max_count: INTEGER)
			-- Initialize a constraint that only allows elements that conform
			-- to type `G' if they have at most `a_max_count' children.
		require
			positive_max_count: a_max_count > 0
		do
			max_count := a_max_count
			has_upper_bound := True
		ensure
			upper_bound_set: has_upper_bound
			max_count_set: max_count = a_max_count
		end

	make_range (a_min_count, a_max_count: INTEGER)
			-- Initialize a constraint that only allows elements that conform
			-- to type `G' if they have at least `a_min_count' and at most
			-- `a_max_count' children.
		require
			positive_min_count: a_min_count > 0
			positive_max_count: a_max_count > 0
			valid_range: a_min_count <= a_max_count
		do
			min_count := a_min_count
			has_lower_bound := True
			max_count := a_max_count
			has_upper_bound := True
		ensure
			lower_bound_set: has_lower_bound
			min_count_set: min_count = a_min_count
			upper_bound_set: has_upper_bound
			max_count_set: max_count = a_max_count
		end

feature {EDL_VISITABLE} -- Visitor

	visit_element (a_element: EDL_ELEMENT)
			-- Set `last_result' to False if `a_element' conforms to `G' and doesn't
			-- have an appropriate number of children.
		do
			if attached {G} a_element as a_compound_element then
				if has_lower_bound and then a_compound_element.count < min_count then
					last_result := False
				end
				if has_upper_bound and then a_compound_element.count > max_count then
					last_result := False
				end
			end
		ensure then
			constraint_violation_not_lost: last_result implies old last_result
		end

	leave_element (a_element: EDL_ELEMENT)
			-- Do nothing.
		do
		ensure then
			constraint_violation_not_lost: last_result implies old last_result
		end

	visit_document (a_document: EDL_DOCUMENT)
			-- Set `last_result' to True.
		do
			last_result := True
		ensure then
			document_fulfills_constraint_by_default: last_result
		end

feature -- Access

	min_count: INTEGER
			-- Minimum number of children for elements that conform
			-- to type `G'.

	max_count: INTEGER
			-- Maximum number of children for elements that conform
			-- to type `G'.

	has_upper_bound: BOOLEAN
			-- Is there a constraint on the maximum number of children?

	has_lower_bound: BOOLEAN
			-- Is there a constraint on the minimum number of children?

invariant

	positive_min_count: has_lower_bound implies min_count > 0
	positive_max_count: has_upper_bound implies max_count > 0
	valid_range: (has_lower_bound and has_upper_bound) implies min_count <= max_count

end
