note
	description: "[
		Objects that generate a human-readable unique string identifier
		for each EDL_IDENTIFIABLE_ELEMENT in a list, based on their names.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_IDENTIFIER_GENERATOR [G -> EDL_IDENTIFIABLE_ELEMENT]

feature -- Basic operations

	generate (a_list: LIST [G]): HASH_TABLE [STRING, G]
			-- Generate a hash table with a human-readable unique string identifier
			-- for each element in `a_list', based on their names.
		require
			names_are_utf8: a_list.for_all (agent (a_identifiable: G): BOOLEAN
				do
					Result := (create {EDL_UTF8_ENCODING}).is_valid (a_identifiable.name)
				end)
		local
			s: STRING
			t: STRING
			i: INTEGER
			list: LINKED_LIST [G]
			empty_list: LINKED_LIST [G]
			table: HASH_TABLE [LINKED_LIST [G], STRING]
			insert: BOOLEAN
			duplicate_list: LINKED_LIST [TUPLE [G,G]]
		do
			create Result.make (a_list.count)

			create table.make (a_list.count)
			create empty_list.make
			create duplicate_list.make

			-- Group elements with the same non-empty name in `table'
			-- "hello" -> <<[ELEMENT 1], [ELEMENT_3]>>
			-- "world" -> <<[ELEMENT 2]>>
			-- Elements with an empty name form a separate list `empty_list'
			from
				a_list.start
			until
				a_list.after
			loop
				s := transformed_name (a_list.item.name)

				if s.is_empty then
					empty_list.put_right (a_list.item)
				else
					if attached table.item (s) as l then
						-- Check if there is already an element such that
						-- `use_same_identifier' is true.
						from
							l.start
							insert := True
						until
							l.after or not insert
						loop
							if a_list.item.use_same_identifier (l.item) then
								insert := False
							else
								l.forth
							end
						end

						if insert then
							-- Insert `item' into the corresponding list.
							l.extend (a_list.item)
						else
							-- Add the element to the duplicate list.
							duplicate_list.extend ([a_list.item, l.item])
						end
					else
						-- Create new list and insert into table.
						create list.make
						list.extend (a_list.item)
						table.put (list, s)
					end
				end

				a_list.forth
			end

			-- For all groups in `table' that only contain one element,
			-- the choice of identifier is clear, identifiers of elements
			-- in the remainig groups will need suffixes.
			from
				table.start
			until
				table.after
			loop
				list := table.item_for_iteration

				if list.count = 1 then
					Result.put (table.key_for_iteration, list.first)
				end

				table.forth
			end

			-- For all remaining groups, use consecutive integers to
			-- generate unique identifiers.
			from
				table.start
			until
				table.after
			loop
				list := table.item_for_iteration
				s := table.key_for_iteration

				from
					list.start
					i := 1
				until
					list.after
				loop
					create t.make_from_string (s)
					t.append_character ('-')
					t.append_integer (i)

					if not Result.has_item (t) then
						Result.put (t, list.item)
						list.forth
					end

					i := i + 1
				end

				table.forth
			end

			-- Insert elements with empty name.
			from
				empty_list.start
				i := 1
			until
				empty_list.after
			loop
				create t.make_empty
				t.append_integer (i)

				if not Result.has_item (t) then
					Result.put (t, empty_list.item)
					empty_list.forth
				end

				i := i + 1
			end

			-- Insert duplicate list.
			from
				duplicate_list.start
			until
				duplicate_list.after
			loop
				if attached {G} duplicate_list.item [1] as g1 and attached {G} duplicate_list.item [2] as g2 then
					Result.put (Result [g2], g1)
				end
				duplicate_list.forth
			end
		ensure
			only_valid_identifiers: attached {LIST [STRING]} Result.linear_representation as identifiers
				and then identifiers.for_all (agent is_valid_identifier)
		end

	is_valid_identifier (a_identifier: STRING): BOOLEAN
			-- Is `a_identifier' a valid identifier?
			-- This is the case if it only contains lowercase alphanumeric
			-- characters or hyphens and is not empty.
		do
			valid_identifier_regex.match (a_identifier)
			Result := valid_identifier_regex.has_matched
		end

feature {NONE} -- Implementation

	transformed_name (a_name: STRING): STRING
			-- Copy of `a_name', transformed as follows:
			-- 1. Replace each uppercase ASCII character by its lowercase equivalent
			-- 2. Replace each space ' ' with a hyphen '-'
			-- 3. Drop each character that is neither a alphanumerical ASCII character nor a hyphen
		require
			name_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_name)
		local
			ascii_name: STRING
			character: CHARACTER
			i: INTEGER
		do
			ascii_name := (create {EDL_UTF8_ENCODING}).to_ascii (a_name)

			create Result.make (ascii_name.count)

			from
				i := 1
			until
				i > ascii_name.count
			loop
				character := ascii_name.item (i)

				if character.is_alpha_numeric then
					Result.append_character (character.as_lower)
				elseif character.is_space or character.is_equal ('-') then
					Result.append_character ('-')
				end

				i := i + 1
			end
		ensure
			is_valid_identifier: is_valid_identifier (Result) or Result.is_empty
		end

	valid_identifier_regex: RX_PCRE_REGULAR_EXPRESSION
			-- Compiled regular expression used to check the validity of an identifier.
		once
			create Result.make

			Result.set_anchored (True)
			Result.set_dollar_endonly (True)
			Result.set_strict (True)
			Result.compile ("[a-z0-9-]+")
		end

end
