note
	description: "[
		Operations for creating an output directory and populating it with content.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_OUTPUT_DIRECTORY

create
	make

feature -- Initialization

	make (a_directory: DIRECTORY)
			-- Set `base_directory' to `a_directory'.
			-- If it already exists, it must be empty and writable.
			-- Otherwise, it is created recursively.
		require
			directory_is_empty_if_exists: a_directory.exists implies a_directory.is_empty
			directory_is_writable_if_exists: a_directory.exists implies a_directory.is_writable
		do
			base_directory := a_directory

			if not base_directory.exists then
				base_directory.recursive_create_dir
			end
		ensure
			base_directory_set: base_directory = a_directory
			directory_exists: a_directory.exists
			directory_writable: a_directory.is_writable
			directory_is_empty: a_directory.is_empty
		end

feature -- Access

	base_directory: DIRECTORY
			-- Directory into wich the created files will be put.

feature -- Miscellaneous

	copy_file (a_file: FILE; a_file_name: STRING)
			-- Copy a file `a_file' into a new file named `a_file_name' relative to the `base_directory'.
			-- If a file with name `a_file_name' is already present, nothing is done.
		require
			a_file_exists: a_file.exists
			a_file_is_readable: a_file.is_readable
		local
			new_file: RAW_FILE
		do
			create new_file.make (absolute_file_name (a_file_name))

			if not new_file.exists then
				create_directory (absolute_file_name (a_file_name))

				new_file.create_read_write

				if a_file.is_closed then
					a_file.open_read
				end

				a_file.copy_to (new_file)
				new_file.close
			end
		end

	write_file (a_file_name: STRING; a_text: STRING)
			-- Create file `a_file_name' in `base_directory' and write `a_text' into this file.
		local
			new_file: RAW_FILE
		do
			create new_file.make_open_write (absolute_file_name (a_file_name))
			new_file.put_string (a_text)
		end

feature {NONE} -- Implementation

	create_directory (a_file_name: EDL_FILE_NAME)
			-- Recursively create any directories that are part of `a_file_name'.
		local
			directory: DIRECTORY
		do
			create directory.make (a_file_name.directory_name)

			if not directory.exists then
				directory.recursive_create_dir
			end
		end

	absolute_file_name (a_file_name: STRING): EDL_FILE_NAME
			-- Absolute name of `a_file_name' (which is relative to `base_directory').
		do
			create Result.make

			Result.set_directory (base_directory.name)
			Result.set_file_name (a_file_name)
		ensure
			result_has_file_name: attached Result.string as s implies s.has_substring (a_file_name)
			is_in_base_directory: attached Result.string as s implies s.starts_with (base_directory.name)
		end

invariant

	base_directory_exists: base_directory.exists
	base_directory_writable: base_directory.is_writable

end
