note
	description: "[
		Translation of documents to LaTeX given a EDL_LATEX_TRANSLATOR_CONTEXT.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_LATEX_TRANSLATOR

inherit
	EDL_TEXT_TRANSLATOR
		redefine
			reset
		end

create
	make_from_context

feature {NONE} -- Initialization

	make_from_context (a_context: EDL_LATEX_TRANSLATOR_CONTEXT)
			-- Initialize with `a_context'.
		do
			reset
			context := a_context
		end

feature {EDL_VISITABLE} -- Visitor

	visit_text_element (a_text_element: EDL_TEXT_ELEMENT)
			-- Append the beginning of the LaTeX representation of `a_text_element'
			-- to `last_result'.
		local
			style: STRING
		do
			if a_text_element.is_superscript then
 				open_command ("textsuperscript")
 			end
			if a_text_element.is_subscript then
 				open_command ("textsubscript")
 				packages.extend ("fixltx2e")
 			end
 			if a_text_element.is_bold or is_text_always_bold > 0 then
 				open_command ("textbf")
 			end
 			if a_text_element.is_italic then
 				open_command ("textit")
 			end
 			if a_text_element.is_underlined then
 				open_command ("underline")
 				packages.extend ("ulem")
 			end

			create style.make_empty

			if attached a_text_element.color as color then
				register_color (color)
				command_with_argument ("textcolor", colors [color])
				append_to_output ("{")
			end

			if attached a_text_element.background_color as color then
				register_color (color)
				command_with_argument ("colorbox", colors [color])
				append_to_output ("{")
			end

 			append_to_output (escape_string (a_text_element.text))
		end

	leave_text_element (a_text_element: EDL_TEXT_ELEMENT)
			-- Append the end of the LaTeX representation of `a_text_element'
			-- to `last_result'.
		do
 			if a_text_element.is_bold or is_text_always_bold > 0 then
  				close_command
  			end
 			if a_text_element.is_italic then
  				close_command
 			end
 			if a_text_element.is_underlined then
 				close_command
 			end
 			if a_text_element.is_subscript then
 				close_command
 			end
 			if a_text_element.is_superscript then
 				close_command
 			end

			if attached a_text_element.background_color as color then
				close_command
			end

 			if attached a_text_element.color as color then
				close_command
			end
		end

	visit_table (a_table: EDL_TABLE)
			-- Append the beginning of the LaTeX representation of `a_table'
			-- to `last_result'.
		local
			s: STRING
			i: INTEGER
		do
			packages.extend ("tabulary")
			create s.make_empty
			from
				i := 1
			until
				i > a_table.column_count
			loop
				s.append ("J@{\hspace{20pt}}")
				if i /= a_table.column_count and a_table.borders_inside_vertical then
					s.append ("|")
				end
				i := i + 1
			end
			if a_table.borders_outside_vertical then
				s.prepend ("|")
				s.append ("|")
			end

			command_with_argument ("vspace", "10pt")
			append_newline_to_output
			command_with_arguments ("setlength", << "\extrarowheight", "1.5pt" >>)
			append_newline_to_output
			open_environment_with_arguments ("tabulary", << "0.9\textwidth", s >>)
			if a_table.borders_outside_horizontal then
				append_line_to_output (simple_command ("hline"))
			end
		end

	leave_table (a_table: EDL_TABLE)
			-- Append the end of the LaTeX representation of `a_table'
			-- to `last_result'.
		do
			if a_table.borders_outside_horizontal then
				append_line_to_output (simple_command ("hline"))
			end
			close_environment ("tabulary")
			command_with_argument ("vspace", "10pt")
			append_newline_to_output
			append_newline_to_output
		end

	visit_table_row (a_table_row: EDL_TABLE_ROW)
			-- Append the beginning of the LaTeX representation of `a_table_row'
			-- to `last_result'.
		do
			append_to_output (whitespaces)
		end

	leave_table_row (a_table_row: EDL_TABLE_ROW)
			-- Append the end of the LaTeX representation of `a_table_row'
			-- to `last_result'.
		do
			append_to_output ("%H%H%N")
			check attached {EDL_TABLE} a_table_row.parent as table then
				if table.borders_inside_horizontal and table.last /= a_table_row then
					append_to_output (simple_command ("hline"))
				end
			end
		end

	visit_table_cell (a_table_cell: EDL_TABLE_CELL)
			-- Append the beginning of the LaTeX representation of `a_table_cell'
			-- to `last_result'.
		do
			if a_table_cell.is_heading then
				is_text_always_bold := is_text_always_bold + 1
			end
		end

	leave_table_cell (a_table_cell: EDL_TABLE_CELL)
			-- Append the end of the LaTeX representation of `a_table_cell'
			-- to `last_result'.
		do
			if a_table_cell.is_heading then
				is_text_always_bold := is_text_always_bold - 1
			end
			check attached a_table_cell.parent as row then
				if row.last /= a_table_cell then
					append_to_output ("&")
				end
			end
		end

	visit_alphabetic_list (a_alphabetic_list: EDL_ALPHABETIC_LIST)
			-- Append the beginning of the LaTeX representation of `a_alphabetic_list'
			-- to `last_result'.
		local
			s, t, u: STRING
		do
			create t.make_from_string ("enumi")
			inspect
				list_nesting_depth
			when 2 then
				t.append ("i")
			when 3 then
				t.append ("ii")
			when 4 then
				t.append ("v")
			else
				-- Nothing to do.
			end
			create s.make_from_string (simple_command ("Alph{"))
			if not a_alphabetic_list.is_upper_case then
				s.to_lower
			end
			s.append (t + "}")
			append_line_to_output ("{%N")
			increment_indentation
			create u.make_from_string ("the" + t)
			command_with_arguments ("renewcommand", <<simple_command (u), s>>)
			append_newline_to_output
			open_environment ("enumerate")
		end

	leave_alphabetic_list (a_alphabetic_list: EDL_ALPHABETIC_LIST)
			-- Append the end of the LaTeX representation of `a_alphabetic_list'
			-- to `last_result'.
		do
			close_environment ("enumerate")
			decrement_indentation
			append_line_to_output ("}")
		end

	visit_arabic_list (a_arabic_list: EDL_ARABIC_LIST)
			-- Append the beginning of the LaTeX representation of `a_arabic_list'
			-- to `last_result'.
		local
			s, t, u: STRING
		do
			create t.make_from_string ("enumi")
			inspect
				list_nesting_depth
			when 2 then
				t.append ("i")
			when 3 then
				t.append ("ii")
			when 4 then
				t.append ("v")
			else
				-- Nothing to do.
			end
			create s.make_from_string (simple_command ("arabic{"))
			s.append (t + "}")
			append_line_to_output ("{%N")
			increment_indentation
			create u.make_from_string ("the" + t)
			command_with_arguments ("renewcommand", <<simple_command (u), s>>)
			append_newline_to_output
			open_environment ("enumerate")
		end

	leave_arabic_list (a_arabic_list: EDL_ARABIC_LIST)
			-- Append the end of the LaTeX representation of `a_arabic_list'
			-- to `last_result'.
		do
			close_environment ("enumerate")
		end

	visit_bullet_list (a_bullet_list: EDL_BULLET_LIST)
			-- Append the beginning of the LaTeX representation of `a_bullet_list'
			-- to `last_result'.
		do
			open_environment ("itemize")
		end

	leave_bullet_list (a_bullet_list: EDL_BULLET_LIST)
			-- Append the end of the LaTeX representation of `a_bullet_list'
			-- to `last_result'.
		do
			close_environment ("itemize")
		end

	visit_roman_list (a_roman_list: EDL_ROMAN_LIST)
			-- Append the beginning of the LaTeX representation of `a_roman_list'
			-- to `last_result'.
		local
			s, t, u: STRING
		do
			create t.make_from_string ("enumi")
			inspect
				list_nesting_depth
			when 2 then
				t.append ("i")
			when 3 then
				t.append ("ii")
			when 4 then
				t.append ("v")
			else
				-- Nothing to do.
			end
			create s.make_from_string (simple_command ("Roman{"))
			if not a_roman_list.is_upper_case then
				s.to_lower
			end
			s.append (t + "}")
			append_line_to_output ("{%N")
			increment_indentation
			create u.make_from_string ("the" + t)
			command_with_arguments ("renewcommand", <<simple_command (u), s>>)
			append_newline_to_output
			open_environment ("enumerate")
		end

	leave_roman_list (a_roman_list: EDL_ROMAN_LIST)
			-- Append the end of the LaTeX representation of `a_roman_list'
			-- to `last_result'.
		do
			close_environment ("enumerate")
		end

	visit_list_item (a_table: EDL_LIST_ITEM)
			-- Append the beginning of the LaTeX representation of `a_table'
			-- to `last_result'.
		do
			append_starting_line_to_output (simple_command ("item"))
		end

	leave_list_item (a_table: EDL_LIST_ITEM)
			-- Append the end of the LaTeX representation of `a_table'
			-- to `last_result'.
		do
			decrement_indentation
			append_newline_to_output
		end

	visit_section (a_section: EDL_SECTION)
			-- Append the beginning of the LaTeX representation of `a_section'
			-- to `last_result'.
		local
			s: STRING
		do
			create s.make_empty
			section_nesting_depth := section_nesting_depth + 1

			inspect section_nesting_depth
			when 1 then
				s.append ("section")
			when 2 then
				s.append ("subsection")
			when 3 then
				s.append ("subsubsection")
			when 4 then
				s.append ("paragraph")
			when 5 then
				s.append ("subparagraph")
			else
				-- This cannot occur.
				check False end
			end

			command_with_argument (s, escape_string (a_section.title))
			increment_indentation
			append_newline_to_output
			append_line_to_output ("\label{" + (context.section_id (a_section) + "}"))
		end

	leave_section (a_section: EDL_SECTION)
			-- Append the end of the LaTeX representation of `a_section'
			-- to `last_result'.
		do
			decrement_indentation
			section_nesting_depth := section_nesting_depth - 1

			if section_nesting_depth = 0 then
				append_line_to_output (simple_command ("pagebreak"))
			end
		end

	visit_image (a_image: EDL_IMAGE)
			-- Append the LaTeX representation of `a_image'
			-- to `last_result'.
		do
			command_with_argument ("includegraphics", context.image_file_name (a_image))
			append_newline_to_output
			packages.extend ("graphicx")
		end

	leave_image (a_image: EDL_IMAGE)
			-- Do nothing.
		do
		end

	visit_paragraph (a_paragraph: EDL_PARAGRAPH)
			-- Append the beginning of the LaTeX representation of `a_paragraph'
			-- to `last_result'.
		do
			if is_paragraph_required (a_paragraph) then
				append_to_output ("%N" + whitespaces)
			end
		end

	leave_paragraph (a_paragraph: EDL_PARAGRAPH)
			-- Append the end of the LaTeX representation of `a_paragraph'
			-- to `last_result'.
		do
			if is_paragraph_required (a_paragraph) then
				append_newline_to_output
				append_newline_to_output
			end
		end

	visit_reference (a_reference: EDL_REFERENCE)
			-- Append the beginning of the LaTeX representation of `a_reference'
			-- to `last_result'.
		local
			s: STRING
		do
			create s.make_empty

			if attached a_reference.destination_uri as uri then
				s.append ("\href{" + ( uri + "}{"))
			elseif attached a_reference.destination_section as section then
				check attached a_reference.parent_document as document and
					attached section.parent_document as section_document then
					if document = section_document then
						s.append ("\hyperref[")
						s.append (context.section_id (section))
						s.append ("]{")
					else
						-- Nothing to do here.
					end
				end
			elseif attached a_reference.destination_document as document then
				-- Nothing to do here.
			end

			append_to_output (s)
		end

	leave_reference (a_reference: EDL_REFERENCE)
			-- Append the end of the LaTeX representation of `a_reference'
			-- to `last_result'.
		local
			s: STRING
		do
			create s.make_empty
			if attached a_reference.destination_uri as uri then
				s := "}"
			elseif attached a_reference.destination_section as section then
				check attached a_reference.parent_document as document and
					attached section.parent_document as section_document then
					if document = section_document then
						s.append ("}")
					else
						s.append ("\footnote{See ")
						s.append (escape_string (section_document.title))
						s.append (", section ")
						s.append (escape_string (section.title))
						s.append ("}")
					end
				end
			elseif attached a_reference.destination_document as document then
				s.append ("\footnote{See ")
				s.append (escape_string (document.title))
				s.append ("}")
			end

			append_to_output (s)
		end

	visit_table_of_contents (a_table_of_contents: EDL_TABLE_OF_CONTENTS)
			-- Append the LaTeX representation of `a_table_of_contents'
			-- to `last_result'.
		do
			append_line_to_output (simple_command ("tableofcontents"))
			append_line_to_output (simple_command ("pagebreak"))
			append_newline_to_output
			append_newline_to_output
		end

	leave_table_of_contents (a_table_of_contents: EDL_TABLE_OF_CONTENTS)
			-- Do nothing.
		do
		end

	visit_raw_element (a_raw_element: EDL_RAW_ELEMENT)
			-- Append content of `a_raw_element' to `last_result'.
		do
			append_line_to_output (a_raw_element.content)
		end

	leave_raw_element (a_raw_element: EDL_RAW_ELEMENT)
			-- Do nothing.
		do
		end

	visit_listing (a_listing: EDL_LISTING)
			-- Append the beginning of the LaTeX representation of `a_listing' to `last_result'.
		local
			lang: STRING
		do
			packages.extend ("listings")

			lang := context.writer.listing_language_mapping.at (a_listing.language)
			if lang.is_equal ("") then
				open_environment ("lstlisting")
			else
				append_starting_line_to_output ("\begin{lstlisting}[language=" + (lang + "]"))
			end

			append_to_output (a_listing.content + "%N")
		end

	leave_listing (a_listing: EDL_LISTING)
			-- Append the end of the LaTeX representation of `a_listing' to `last_result'.
		do
			close_environment ("lstlisting")
		end

	visit_document (a_document: EDL_DOCUMENT)
			-- Append some of the beginning of the LaTeX representation of
			-- `a_document' to `last_result'.
		local
			s: STRING
		do
			open_environment ("document")

			packages.extend ("hyperref")
			packages.extend ("geometry")
			packages.extend ("xcolor")
			packages.extend ("enumitem")

			command_with_argument ("title", escape_string (a_document.title))
			append_newline_to_output

			if attached a_document.author as a then
				command_with_argument ("author", escape_string (a))
				append_newline_to_output
			end

			if attached a_document.date as d then
				create s.make_empty
				s.append (d.year.out)
				s.append ("-")
				if d.month < 10 then
					s.append ("0")
				end
				s.append (d.month.out)
				s.append ("-")
				if d.day < 10 then
					s.append ("0")
				end
				s.append (d.day.out)

				command_with_argument ("date", s)
				append_newline_to_output
			end

			append_line_to_output (simple_command ("maketitle"))
			section_nesting_depth := 0
			list_nesting_depth := 0
		end

	leave_document (a_document: EDL_DOCUMENT)
			-- Append the end of the LaTeX representation of `a_document'
			-- to `last_result', and prepend it with the beginning.
		local
			s: STRING
		do
			create s.make_empty
			s.append ("%Hdocumentclass[")
			s.append (context.writer.page_dimension)
			s.to_lower
			s.append ("paper")
			if context.writer.has_title_page then
				s.append (",titlepage")
			end
			if context.writer.is_portrait then
				s.append (",portrait")
			else
				s.append (",landscape")
			end
			s.append ("]{article}%N%N")

			from
				packages.start
			until
				packages.after
			loop
				s.append (simple_command ("usepackage") + "{")
				s.append (packages.item)
				s.append ("}%N")
				packages.forth
			end

			s.append ("%N")

			s.append ("%Hrenewcommand{%Hlabelitemi}{$%Hbullet$}%N")
			s.append ("%Hrenewcommand{%Hlabelitemii}{$%Hbullet$}%N")
			s.append ("%Hrenewcommand{%Hlabelitemiii}{$%Hbullet$}%N")
			s.append ("%Hrenewcommand{%Hlabelitemiv}{$%Hbullet$}%N")

			s.append ("%N")

			s.append ("\geometry{a4paper, left=25mm, right=25mm, top=25mm, bottom=25mm}")
			s.append ("%N%N")

			s.append (listings_configuration)
			s.append ("%N")

			s.append (hyperref_configuration)
			s.append ("%N")

			s.append ("\setitemize{noitemsep,topsep=2pt}%N")
			s.append ("\setenumerate{noitemsep,topsep=2pt}%N")
			s.append ("%N")

			from
				colors.start
			until
				colors.after
			loop
				s.append ("\definecolor{" + (colors.item_for_iteration + "}{RGB}{"))
				s.append (colors.key_for_iteration.red.out)
				s.append (",")
				s.append (colors.key_for_iteration.green.out)
				s.append (",")
				s.append (colors.key_for_iteration.blue.out)
				s.append ("}%N")

				colors.forth
			end

			s.append ("%Husepackage[utf8x]{inputenc}")
			s.append ("%N%N%N")

			last_result.prepend (s)

			close_environment ("document")
		end

feature {NONE} -- Implementation

	is_paragraph_required (a_paragraph: EDL_PARAGRAPH): BOOLEAN
			-- Is the paragraph represenentation required for `a_paragraph'?
		do
			if not attached {EDL_TABLE_CELL} a_paragraph.parent and not attached {EDL_LIST_ITEM} a_paragraph.parent then
				Result := True
			end
		end

	escape_string (a_string: STRING): STRING
			-- Prepare `a_string' for output in a LaTeX file by escaping
			-- the following characters: # $ % ^ & _ { } ~ \
		do
			create Result.make_from_string (a_string)
			Result.replace_substring_all ("%H", "%Htextbackslash")
			Result.replace_substring_all ("#", "%H")
			Result.replace_substring_all ("$", "%H$")
			Result.replace_substring_all ("%%", "%H%%")
			Result.replace_substring_all ("^", "%Htextasciicircum")
			Result.replace_substring_all ("&", "%H&")
			Result.replace_substring_all ("_", "%H_")
			Result.replace_substring_all ("{", "%H{")
			Result.replace_substring_all ("}", "%H}")
			Result.replace_substring_all ("~", "%H~")
		end

	reset
			-- Reset internal state.
		do
			Precursor

			create packages.make
			packages.compare_objects

			create colors.make (0)
		ensure then
			packages_is_empty: packages.is_empty
		end

	colors: HASH_TABLE [STRING, EDL_COLOR]
			-- Mapping of colors to color names used in the output.

	register_color (a_color: EDL_COLOR)
			-- Add `a_color' to `colors' if its not registered yet
			-- and associate it with its color name.
		local
			s: STRING
		do
			packages.extend ("color")

			if not colors.has_key (a_color) then
				if attached a_color.name as color_name then
					colors.put (color_name, a_color)
				else
					s := "color-"
					s.append_integer (colors.count + 1)

					colors.put (s, a_color)
				end
			end
		end

	packages: LINKED_SET [STRING]
			-- Set of packages needed for the current document.

	section_nesting_depth: INTEGER
			-- Current section nesting depth.

	list_nesting_depth: INTEGER
			-- Current list nesting depth.

	is_text_always_bold: INTEGER
			-- Are text elements always bold? 0 means no, any
			-- other number yes.

	context: EDL_LATEX_TRANSLATOR_CONTEXT
			-- Current context.

	simple_command (a_command: STRING): STRING
			-- `a_command' preceded by a backslash.
		do
			Result := a_command
			Result.prepend ("%H")
		end

	open_command (a_command: STRING)
			-- Append `a_command' to `last_result'.
		do
			append_to_output ("%H" + (a_command + "{"))
		end

	command_with_argument (a_command, a_argument: STRING)
			-- Append `a_command' with `a_argument' as argument to `last_result'.
		do
			command_with_arguments (a_command, <<a_argument>>)
		end

	command_with_arguments (a_command: STRING; a_arguments: INDEXABLE [STRING, INTEGER])
			-- Append `a_command' with `a_arguments' as arguments to `last_result'.
		local
			s: STRING
			i: INTEGER
		do
			create s.make_empty
			check attached a_arguments.index_set as index_set then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				from
					i := index_set.lower
				until
					i > index_set.upper
				loop
					s.append ("{")
					s.append (a_arguments.item (i))
					s.append ("}")
					i := i + 1
				end
			end
			append_to_output ("%H" + a_command )
			append_to_output (s)
		end

	close_command
			-- Append } to `last_result'.
		do
			append_to_output ("}")
		end

	open_environment (a_environment: STRING)
			-- Append the beginning of `a_environment' to `last_result'.
		do
			append_starting_line_to_output ("%Hbegin{" + (a_environment + "}"))
		end

	open_environment_with_argument (a_environment, a_argument: STRING)
			-- Append the beginning of `a_environment' with `a_argument' as argument
			-- to `last_result'.
		do
			open_environment_with_arguments (a_environment, <<a_argument>>)
		end

	open_environment_with_arguments (a_environment: STRING; a_arguments: INDEXABLE [STRING, INTEGER])
			-- Append the beginning of `a_environment' with `a_arguments' as
			-- arguments to `last_result'.
		local
			s: STRING
			i: INTEGER
		do
			create s.make_empty
			s.append ("%Hbegin{" + (a_environment + "}"))
			check attached a_arguments.index_set as index_set then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				from
					i := index_set.lower
				until
					i > index_set.upper
				loop
					s.append ("{")
					s.append (a_arguments.item (i))
					s.append ("}")
					i := i + 1
				end
			end
			append_starting_line_to_output (s)
		end

	close_environment (a_environment: STRING)
			-- Append the end of `a_environment' to `last_result'.
		do
			append_ending_line_to_output ("%Hend{" + (a_environment + "}"))
			append_to_output ("%N%N")
		end

	whitespaces: STRING
			-- Returns the current number of whitespaces necessary.
		do
			create Result.make_filled (whitespace, indentation)
		end

	whitespace: CHARACTER = '%T'
			-- Whitespace character used to indent the output.

	indentation: INTEGER
			-- Current indentation level.

	increment_indentation
			-- Increment the indendation level by one.
		do
			indentation := indentation + 1
		ensure
			incremented: old indentation + 1 = indentation
		end

	decrement_indentation
			-- Ddecrement the indendation level by one.
		require
			positiv_indentation: indentation > 0
		do
			indentation := indentation - 1
		ensure
			incremented: old indentation - 1 = indentation
		end

	append_to_output (a_string: detachable STRING)
			-- Append `a_string' to `last_result'.
		do
			last_result.append (a_string)
		end

	append_newline_to_output
			-- Append a newline to `last_result'.
		do
			append_to_output ("%N")
		end

	append_line_to_output (a_string: detachable STRING)
			-- Append `a_string' to `last_result' and start a new line afterwards.
		do
			last_result.append (whitespaces + a_string)
			last_result.append ("%N")
		end

	append_starting_line_to_output (a_string: detachable STRING)
			-- Append `a_string' to `last_result', start a new line and increment
			-- the indentation.
		do
			append_line_to_output (a_string)
			increment_indentation
		end

	append_ending_line_to_output (a_string: detachable STRING)
			-- Decrement the indentation, append `a_string' to `last_result' and
			-- start a new line.
		do
			decrement_indentation
			append_line_to_output (a_string)
		end

	listings_configuration: STRING
			-- Configuration string for listings package.
		once
			create Result.make_empty
			Result.append ("\definecolor{lst-darkgreen}{RGB}{0,127,14}%N")
			Result.append ("\lstset{%N")
			Result.append ("%Tbasicstyle=\ttfamily\small,%N")
			Result.append ("%Tidentifierstyle=\mdseries,%N")
			Result.append ("%Tstringstyle=\color{gray},%N")
			Result.append ("%Tnumbers=left,%N")
			Result.append ("%Tnumbersep=5pt,%N")
			Result.append ("%Tinputencoding=utf8x,%N")
			Result.append ("%Txleftmargin=10pt,%N")
			Result.append ("%Txrightmargin=10pt,%N")
			Result.append ("%Tkeywordstyle=[1]\bfseries\color{blue},%N")
			Result.append ("%Tkeywordstyle=[2]\bfseries\color{teal},%N")
			Result.append ("%Tkeywordstyle=[3]\bfseries\color{red},%N")
			Result.append ("%Tkeywordstyle=[4]\bfseries\color{yellow},%N")
			Result.append ("%Tnumberstyle=\tiny,%N")
			Result.append ("%Tstepnumber=1,%N")
			Result.append ("%Tbreaklines=true,%N")
			Result.append ("%Tframe=tlrb,%N")
			Result.append ("%Tshowstringspaces=false,%N")
			Result.append ("%Ttabsize=4,%N")
			Result.append ("%Tcommentstyle=\color{lst-darkgreen},%T%N")
			Result.append ("%Tcaptionpos=b,%N")
			Result.append ("%Tfloat=float,%N")
			Result.append ("%Tlanguage={Eiffel}%N")
			Result.append ("}%N")
		end

	hyperref_configuration: STRING
			-- Configuration string for hyperref package.
		once
			create Result.make_empty
			Result.append ("\definecolor{hyper-darkblue}{RGB}{0,0,120}%N")
			Result.append ("\hypersetup{%N")
			Result.append ("%Tcolorlinks=true,%N")
			Result.append ("%Tlinkcolor=hyper-darkblue,%N")
			Result.append ("%Tcitecolor=hyper-darkblue,%N")
			Result.append ("%Tfilecolor=hyper-darkblue,%N")
			Result.append ("%Turlcolor=hyper-darkblue%N")
			Result.append ("}%N")
		end

end
