note
	description: "[
		Objects that provide combined information about all documents in a document 
		collection and manage the names of files created in the output directory
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_TEXT_TRANSLATOR_CONTEXT

feature {NONE} -- Initialization

	make (a_document_collection: EDL_DOCUMENT_COLLECTION)
			-- Initialize with `a_document_collection', collecting any information
			-- that is needed later on.
		local
			document_identifier_generator: EDL_IDENTIFIER_GENERATOR [EDL_DOCUMENT]
			section_identifier_generator: EDL_IDENTIFIER_GENERATOR [EDL_SECTION]
			image_identifier_generator: EDL_IDENTIFIER_GENERATOR [EDL_IMAGE]
			section_collector: EDL_ELEMENT_COLLECTOR [EDL_SECTION]
			image_collector: EDL_ELEMENT_COLLECTOR [EDL_IMAGE]
		do
			document_collection := a_document_collection

			-- Generate the document identifiers.
			create document_identifier_generator
			document_identifiers := document_identifier_generator.generate (document_collection)

			-- Generate a hash table for the section identifiers of all documents.
			from
				create section_identifiers.make (document_collection.count * estimated_sections) -- Avoid immediate resizing.
				create section_identifier_generator
				create section_collector
				document_collection.start
			until
				document_collection.after
			loop
				section_collector.collect (document_collection.item)
				section_identifiers.merge (section_identifier_generator.generate (section_collector.last_list))
				document_collection.forth
			end

			-- Collect the images in all documents and generate identifiers for them.
			from
				create images.make
				create image_collector
				document_collection.start
			until
				document_collection.after
			loop
				image_collector.collect (document_collection.item)
				images.append (image_collector.last_list)
				document_collection.forth
			end

			create image_identifier_generator
			image_identifiers := image_identifier_generator.generate (images)
		ensure
			document_collection_set: document_collection = a_document_collection
		end

feature -- Access

	document_file_name (a_document: EDL_DOCUMENT): FILE_NAME
			-- Unique file name of `a_document'.
		require
			in_document_collection: document_collection.has (a_document)
		do
			create Result.make

			Result.set_file_name (document_identifiers[a_document])
			Result.add_extension (document_file_extension)
		ensure
			non_empty_result: not Result.is_empty
		end

	section_id (a_section: EDL_SECTION): STRING
			-- Unique section ID of `a_section'.
		require
			section_is_in_document_collection: attached a_section.parent_document as document and then
				document_collection.has (document)
		do
			Result := section_identifiers[a_section]
		ensure
			non_empty_result: not Result.is_empty
		end

	image_file_name (a_image: EDL_IMAGE): FILE_NAME
			-- File name of `a_image' relative to the output direcotry.
		require
			image_is_in_document_collection: attached a_image.parent_document as document and then
				document_collection.has (document)
		do
			create Result.make

			Result.set_file_name ("images/" + image_identifiers[a_image])
			Result.add_extension (a_image.file_name.extension)
		ensure
			non_empty_result: not Result.is_empty
		end

	document_collection: EDL_DOCUMENT_COLLECTION
			-- Current document collection.

	images: LINKED_LIST [EDL_IMAGE]
			-- List of all images in all documents of the current document
			-- collection.

feature {NONE} -- Implementation

	estimated_sections: INTEGER = 10
			-- Estimated number of sections in an average document.

	document_file_extension: STRING
			-- File extension of documents.
		deferred
		end

	document_identifiers: HASH_TABLE [STRING, EDL_DOCUMENT]
			-- Identifiers for all documents in the current document collection.

	section_identifiers: HASH_TABLE [STRING, EDL_SECTION]
			-- Identifiers for all sections of all documents in the current
			-- document collection.

	image_identifiers: HASH_TABLE [STRING, EDL_IMAGE]
			-- Identifiers for all images of all documents in the current
			-- document collection.

end
