note
	description: "[
		Tables are used to represent two-dimensionally structured data. Tables may have an
		arbitrary number of rows and an arbitrary number of columns. A cell, representing the
		intersection of a single row and a single column, may contain autonomous elements.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_TABLE

inherit
	EDL_COMPOUND_ELEMENT [EDL_TABLE_ROW]
		rename
			make_from_element as make_from_row,
			make_from_elements as make_from_rows,
			extend_with_elements as extend_with_rows
		redefine
			is_valid_child,
			are_valid_children,
			default_create
		end

	EDL_AUTONOMOUS_ELEMENT
		undefine
			default_create,
			has
		end

create
	default_create,
	make_from_row,
	make_from_rows,
	make_simple,
	make_simple_with_header

feature {NONE} -- Initialization

	default_create
			-- Initialize without children, and full borders.
		do
			Precursor {EDL_COMPOUND_ELEMENT}
			borders_inside_horizontal := True
			borders_inside_vertical := True
			borders_outside_horizontal := True
			borders_outside_vertical := True
		ensure then
			borders_on_the_inside: borders_inside_horizontal and borders_inside_vertical
			borders_on_the_outside: borders_outside_horizontal and borders_outside_vertical
		end

	make_simple (a_string_matrix: INDEXABLE [INDEXABLE [STRING, INTEGER], INTEGER])
			-- Initialize with `a_string_matrix', placing each string in
			-- a table cell.
		require
			valid_matrix: is_rectangular (a_string_matrix)
			matrix_is_utf8: is_string_matrix_utf8 (a_string_matrix)
		do
			default_create
			extend_with_string_matrix (a_string_matrix, agent (a_strings: INDEXABLE [STRING, INTEGER]): EDL_TABLE_ROW
				do
					create Result.make_simple (a_strings)
				end)
		ensure
			correct_count: attached a_string_matrix.index_set as index_set implies count = index_set.count
			borders_on_the_inside: borders_inside_horizontal and borders_inside_vertical
			borders_on_the_outside: borders_outside_horizontal and borders_outside_vertical
		end

	make_simple_with_header (a_header: INDEXABLE [STRING, INTEGER]; a_string_matrix: INDEXABLE [INDEXABLE [STRING, INTEGER], INTEGER])
			-- Initialize with `a_string_matrix' and a `a_header',
			-- placing each string in in a table cell, where the strings
			-- from `a_header' are placed in heading table cells.
		require
			valid_matrix: is_rectangular (a_string_matrix)
			valid_header_count: attached a_header.index_set as set implies (set.count = count_of_first_row (a_string_matrix))
			header_is_utf8: are_strings_utf8 (a_header)
			matrix_is_utf8: is_string_matrix_utf8 (a_string_matrix)
		do
			default_create
			extend (create {EDL_TABLE_ROW}.make_simple_header (a_header))
			extend_with_string_matrix (a_string_matrix, agent (a_strings: INDEXABLE [STRING, INTEGER]): EDL_TABLE_ROW
				do
					create Result.make_simple (a_strings)
				end)
		ensure
			correct_count: attached a_string_matrix.index_set as index_set implies count = index_set.count + 1
			borders_on_the_inside: borders_inside_horizontal and borders_inside_vertical
			borders_on_the_outside: borders_outside_horizontal and borders_outside_vertical
		end

feature -- Access

	borders_inside_horizontal: BOOLEAN assign set_borders_inside_horizontal
			-- Does the table have horizontal borders between consecutive rows?

	borders_inside_vertical: BOOLEAN assign set_borders_inside_vertical
			-- Does the table have vertical borders between consecutive rows?

	borders_outside_horizontal: BOOLEAN assign set_borders_outside_horizontal
			-- Does the table have horizontal borders before the first and after
			-- the last row?

	borders_outside_vertical: BOOLEAN assign set_borders_outside_vertical
			-- Does the table have vertical borders on the left and right side?

feature -- Measurement

	column_count: INTEGER
			-- Number of columns.
		require
			not_empty: count > 0
		do
			Result := children.first.count
		ensure
			non_negative: Result >= 0
		end

feature -- Element change

	extend_simple (a_strings: INDEXABLE [STRING, INTEGER])
			-- Add one row containing the strings from `a_strings', each in
			-- a table cell.
		require
			valid_row: count = 0 or else (attached a_strings.index_set as index_set implies column_count = index_set.count)
			strings_are_utf8: are_strings_utf8 (a_strings)
		do
			extend (create {EDL_TABLE_ROW}.make_simple (a_strings))
		ensure
			correct_count: count = old count + 1
		end

	set_borders_inside_horizontal (a_boolean: BOOLEAN)
			-- Change `borders_inside_horizontal' to `a_boolean'.
		do
			borders_inside_horizontal := a_boolean
		ensure
			value_set: borders_inside_horizontal = a_boolean
		end

	set_borders_inside_vertical (a_boolean: BOOLEAN)
			-- Change `borders_inside_vertical' to `a_boolean'.
		do
			borders_inside_vertical := a_boolean
		ensure
			value_set: borders_inside_vertical = a_boolean
		end

	set_borders_outside_horizontal (a_boolean: BOOLEAN)
			-- Change `borders_outside_horizontal' to `a_boolean'.
		do
			borders_outside_horizontal := a_boolean
		ensure
			value_set: borders_outside_horizontal = a_boolean
		end

	set_borders_outside_vertical (a_boolean: BOOLEAN)
			-- Change `borders_outside_vertical' to `a_boolean'.
		do
			borders_outside_vertical := a_boolean
		ensure
			value_set: borders_outside_vertical = a_boolean
		end

feature -- Miscellaneous

	is_valid_child (a_child: EDL_TABLE_ROW): BOOLEAN
			-- Is it ok to add `a_child'? Answer: If `a_child' has the same number of
			-- children as the first child, or if the table has no children yet.
		do
			Result := count = 0 or else a_child.count = first.count
		ensure then
			definition: Result = (count = 0 or else a_child.count = first.count)
		end

	are_valid_children (a_elements: ARRAY [EDL_TABLE_ROW]): BOOLEAN
			-- Is it ok to add `a_elements'?
		do
			Result := Precursor (a_elements)
			Result := Result and are_of_same_length (a_elements)
		ensure then
			valid_only_if_same_length: Result implies are_of_same_length (a_elements)
		end

	are_of_same_length (a_rows: ARRAY [EDL_TABLE_ROW]): BOOLEAN
			-- Are the rows in `a_rows' of the same length?
		local
			i, required_count: INTEGER
		do
			Result := True

			check attached a_rows.index_set as index_set then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				if index_set.count > 0 then
					required_count := a_rows.item (index_set.lower).count

					from
						i := index_set.lower
					until
						i > index_set.upper or not Result
					loop
						Result := a_rows.item (i).count = required_count
						i := i + 1
					end
				end
			end
		end

	is_rectangular (a_string_matrix: INDEXABLE [INDEXABLE [STRING, INTEGER], INTEGER]): BOOLEAN
			-- Is `a_string_matrix' rectangular, meaning that all lists in the
			-- list `a_string_matrix' have the same number of children.
		local
			i, required_count: INTEGER
		do
			Result := True

			check attached a_string_matrix.index_set as index_set then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				if index_set.count > 0 then
					required_count := count_of_first_row (a_string_matrix)

					from
						i := index_set.lower
					until
						i > index_set.upper or not Result
					loop
						if attached a_string_matrix.item (i).index_set as index_set_1 then
							Result := index_set_1.count = required_count
						end
						i := i + 1
					end
				end
			end
		end

	count_of_first_row (a_string_matrix: INDEXABLE [INDEXABLE [STRING, INTEGER], INTEGER]): INTEGER
			-- What is the number of items in the first row of `a_string_matrix'?
		do
			check attached a_string_matrix.index_set as index_set then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				if index_set.count > 0 then
					check attached a_string_matrix.item (index_set.lower).index_set as index_set_1 then
						-- Again, always true.
						Result := index_set_1.count
					end
				end
			end
		end

	is_string_matrix_utf8 (a_string_matrix: INDEXABLE [INDEXABLE [STRING, INTEGER], INTEGER]): BOOLEAN
			-- Are all strings in `a_string_matrix' valid UTF-8 encoded?
		local
			utf8_encoding: EDL_UTF8_ENCODING
			i: INTEGER
		do
			Result := True

			check attached a_string_matrix.index_set as interval then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				from
					create utf8_encoding
					i := interval.lower
				until
					i > interval.upper or not Result
				loop
					Result := are_strings_utf8 (a_string_matrix.item (i))
					i := i + 1
				end
			end
		end

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor' and pass it on to the children.
		do
			a_visitor.visit_table (Current)
			accept_children (a_visitor)
			a_visitor.leave_table (Current)
		end

feature {NONE} -- Implementation

	extend_with_string_matrix (a_string_matrix: INDEXABLE [INDEXABLE [STRING, INTEGER], INTEGER]; a_transformation: FUNCTION [ANY, TUPLE [INDEXABLE [STRING, INTEGER]], EDL_TABLE_ROW])
		local
			i: INTEGER
		do
			check attached a_string_matrix.index_set as interval then
					-- Is always true since `index_set' has a post-condition that
					-- prevents it from being Void.
				from
					i := interval.lower
				until
					i > interval.upper
				loop
					extend (a_transformation.item ([a_string_matrix.item (i)]))
					i := i + 1
				end
			end
		ensure
			correct_count: attached a_string_matrix.index_set as index_set implies count = old count + index_set.count
		end

end
