note
	description: "Summary description for {EDL_HTML_TAG}."
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_HTML_TAG

inherit
	EDL_HTML_NODE

	LINKED_LIST [EDL_HTML_NODE]
		rename
			make as ll_make,
			is_equal as ll_is_equal,
			copy as ll_copy
		select
			ll_is_equal,
			ll_copy
		end

create
	make,
	make_with_attribute,
	make_with_text

convert
	make ({STRING}),
	out_inline: {STRING}

feature {NONE} -- Initialization

	make (a_tag: STRING)
			-- Initialize a new HTML tag called `a_tag'.
		require
			non_empty_tag: not a_tag.is_empty
		do
			ll_make
			has_closing_tag := True
			tag := a_tag
			create attributes.make (0)
		ensure
			tag_set: tag = a_tag
			correct_count: count = 0
			correct_attributes_count: attributes.count = 0
			is_not_inline: not is_inline
			has_closing_tag: has_closing_tag
		end

	make_with_attribute (a_tag, a_key, a_value: STRING)
			-- Initialize a new HTML tag called `a_tag' with an attribute
			-- `a_key' with a value `a_value'.
		require
			non_empty_tag: not a_tag.is_empty
			non_empty_key: not a_key.is_empty
		do
			make (a_tag)
			attributes.put (a_value, a_key)
		ensure
			tag_set: tag = a_tag
			correct_count: count = 0
			correct_attributes_count: attributes.count = 1
			attribute_set: attributes.item (a_key).is_equal (a_value)
			is_not_inline: not is_inline
			has_closing_tag: has_closing_tag
		end

	make_with_text (a_tag: STRING; a_text: STRING)
			-- Initialize a new HTML tag called `a_tag' with an HTML text node containing
			-- `a_text'.
		require
			non_empty_tag: not a_tag.is_empty
		do
			make (a_tag)
			extend_with_text (a_text)
		ensure
			tag_set: tag = a_tag
			correct_count: count = 1
			correct_attributes_count: attributes.count = 0
			is_not_inline: not is_inline
			has_closing_tag: has_closing_tag
		end

feature -- Access

	out_indented (a_indentation_level: INTEGER): STRING
			-- Indented string representation.
		do
			Result := whitespaces (a_indentation_level)

			if is_inline or for_all (agent (a_node: EDL_HTML_NODE): BOOLEAN do
					Result := attached {EDL_HTML_TEXT_NODE} a_node
				end) then
				Result.append (out_inline)
			else
				Result.append (opening_tag)

				if has_closing_tag then
					Result.append_character ('%N')

					from
						start
					until
						after
					loop
						Result.append (item.out_indented (a_indentation_level + 1))
						Result.append_character ('%N')
						forth
					end

					Result.append (whitespaces (a_indentation_level))
					Result.append (closing_tag)
				end
			end
		end

	out_inline: STRING
			-- Inline string representation.
		do
			create Result.make_empty

			Result.append (opening_tag)

			if has_closing_tag then
				from
					start
				until
					after
				loop
					Result.append (item.out_inline)
					forth
				end

				Result.append (closing_tag)
			end
		end

	tag: STRING
			-- Name of the tag.

	attributes: HASH_TABLE [STRING, STRING]
			-- HTML attributes of the tag.

	is_inline: BOOLEAN assign set_inline
			-- Is the tag used inline?

	has_closing_tag: BOOLEAN assign set_closing_tag
			-- Does the tag have a closing tag?

feature -- Element change

	extend_with_text (a_text: STRING)
			-- Append an HTML text node containing `a_text'.
		do
			extend (create {EDL_HTML_TEXT_NODE}.make (a_text))
		ensure
			correct_count: count = old count + 1
		end

	extend_with_unescaped_text (a_unescaped_text: STRING)
			-- Append an HTML text node containing `a_text' unescaped.
		do
			extend (create {EDL_HTML_TEXT_NODE}.make_unescaped (a_unescaped_text))
		ensure
			correct_count: count = old count + 1
		end

	set_style (a_mapping: HASH_TABLE [STRING, STRING])
			-- Update style attribute as specified in `a_mapping'.
		local
			value: STRING
		do
			create value.make_empty

			from
				a_mapping.start
			until
				a_mapping.after
			loop
				value.append (a_mapping.key_for_iteration)
				value.append (": ")
				value.append (a_mapping.item_for_iteration)
				value.append ("; ")

				a_mapping.forth
			end

			value.prune_all_trailing (' ')

			-- Keep existing style.
			if attributes.has ("style") then
				create value.make_from_string (value + (" " + attributes ["style"]))
			end

			attributes.force (value, "style")
		ensure
			has_style_key: attributes.has_key ("style")
		end

	set_inline (a_boolean: BOOLEAN)
			-- Set `is_inline' to `a_boolean'.
		do
			is_inline := a_boolean
		ensure
			is_inline_set: is_inline = a_boolean
		end

	set_closing_tag (a_boolean: BOOLEAN)
			-- Set `has_closing_tag' to `a_boolean'.
		require
			tag_is_empty: a_boolean implies is_empty
		do
			has_closing_tag := a_boolean
		ensure
			has_closing_tag_set: has_closing_tag = a_boolean
		end

	add_class (a_class: STRING)
			-- Add `a_class' to the class attribute of this tag.
		require
			non_empty_class: not a_class.is_empty
		local
			current_classes: STRING
		do
			create current_classes.make_empty

			if attributes.has_key ("class") then
				current_classes := attributes.item ("class")
			end

			current_classes.append (" ")
			current_classes.append (a_class)
			current_classes.prune_all_leading (' ')

			attributes.put (current_classes, "class")
		ensure
			has_class_attributes: attributes.has_key ("class")
			has_class: attributes.item ("class").has_substring (a_class)
		end

feature -- Miscellaneous

	encoded_uri (a_uri: STRING): STRING
			-- URI `a_uri' encoded for HTML.
		do
			create Result.make_from_string (a_uri)

			from
				uri_encoding_characters.start
			until
				uri_encoding_characters.after
			loop
				Result.replace_substring_all (
					uri_encoding_characters.key_for_iteration,
					uri_encoding_characters.item_for_iteration)
				uri_encoding_characters.forth
			end
		end


feature {NONE} -- Implementation

	opening_tag: STRING
			-- String representation of the opening tag.
		do
			create Result.make_empty

			Result.append_character ('<')
			Result.append (tag)

			from
				attributes.start
			until
				attributes.after
			loop
				if not attributes.item_for_iteration.is_empty then
					Result.append_character (' ')
					Result.append (attributes.key_for_iteration)
					Result.append ("=%"")

					if attributes.key_for_iteration.is_equal ("src") or attributes.key_for_iteration.is_equal ("href") then
						Result.append (escaped_text (encoded_uri (attributes.item_for_iteration)))
					else
						Result.append (escaped_text (attributes.item_for_iteration))
					end

					Result.append ("%"")
				end

				attributes.forth
			end

			Result.append_character ('>')
		end

	closing_tag: STRING
			-- String representation of the closing tag.
		do
			create Result.make_empty

			Result.append ("</")
			Result.append (tag)
			Result.append (">")
		end

	uri_encoding_characters: HASH_TABLE [STRING, STRING]
			-- Mapping of characters to encoded URI characters.
		local
			i: NATURAL
			s1, s2: STRING
		once
			create Result.make (50)

			Result.put ("%%25", "%%")
			Result.put ("%%20", " ")
			Result.put ("%%24", "$")
			Result.put ("%%2B", "+")
			Result.put ("%%3B", ";")
			Result.put ("%%40", "@")
			Result.put ("%%22", "%"")
			Result.put ("%%3C", "<")
			Result.put ("%%3E", ">")
			Result.put ("%%7B", "{")
			Result.put ("%%7D", "}")
			Result.put ("%%7C", "|")
			Result.put ("%%5C", "\")
			Result.put ("%%5E", "^")
			Result.put ("%%7E", "~")
			Result.put ("%%5B", "[")
			Result.put ("%%5D", "]")
			Result.put ("%%60", "`")

			-- ASCII control characters
			from
				i := 0
			until
				i > 31
			loop
				create s1.make_empty
				s1.append_code (i)
				create s2.make_from_string (i.to_hex_string)
				create s2.make_from_string ("%%" + s2.substring (s2.count - 1, s2.count))
				Result.put (s2, s1)
				i := i + 1
			end
		end

invariant

	non_empty_tag: not tag.is_empty
	empty_if_no_closing_tag: not has_closing_tag implies is_empty

end
