note
	description: "[
		Facilities that output documents or document collections to a directory
		as LaTeX files, alongside potential user-defined images.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_LATEX_WRITER

inherit
	EDL_WRITER
		redefine
			document_fulfills_all_constraints,
			output_document_collection,
			supports_listing_language,
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Initialize with A4, portrait and a title page.
		do
			Precursor
			is_portrait := True
			page_dimension := "A4"
			has_title_page := True
		ensure then
			is_portrait: is_portrait
			a4_page_dimension: page_dimension.is_equal ("A4")
			has_title_page: has_title_page
		end

feature -- Access

	is_portrait: BOOLEAN
			-- Is the page orientation protrait?

	has_title_page: BOOLEAN
			-- Do the documents have a title page?

	page_dimension: STRING
			-- Page dimension, one of `valid_page_dimensions'.

	valid_page_dimensions: SET [STRING]
			-- Valid page dimensions (upper-case).
			-- A0, A1, A2, A3, A4, A5, A6, B0, B1, B2, B3, B4, B5, B6
		once
			create {LINKED_SET [STRING]} Result.make
			Result.extend ("A0")
			Result.extend ("A1")
			Result.extend ("A2")
			Result.extend ("A3")
			Result.extend ("A4")
			Result.extend ("A5")
			Result.extend ("A6")
			Result.extend ("B0")
			Result.extend ("B1")
			Result.extend ("B2")
			Result.extend ("B3")
			Result.extend ("B4")
			Result.extend ("B5")
			Result.extend ("B6")
		end

feature -- Measurement

	document_fulfills_all_constraints (a_document: EDL_DOCUMENT): BOOLEAN
			-- Does `a_document' fulfill all requirements of the LaTeX writer?
		do
			Result := Precursor (a_document)

			-- Valid section nesting
			Result := Result and a_document.fulfills_constraint (create {EDL_NESTING_DEPTH_CONSTRAINT [EDL_SECTION]}.make (maximum_section_nesting))

			-- Valid list nesting
			Result := Result and a_document.fulfills_constraint (create {EDL_NESTING_DEPTH_CONSTRAINT [EDL_LIST]}.make (maximum_list_nesting))
		end

	maximum_section_nesting: INTEGER = 5
			-- Maximum section nesting supported by LaTeX.

	maximum_list_nesting: INTEGER = 5
			-- Maximum list nesting supported by LaTeX.

	supports_listing_language (a_language: STRING): BOOLEAN
			-- Is `a_language' supported for listing elements?
		do
			Result := listing_language_mapping.has (a_language)
		end

feature -- Element change

	set_page_dimension (a_page_dimension: STRING)
			-- Change `page_dimension' to `a_page_dimension'.
		require
			valid_page_dimension: valid_page_dimensions.has (a_page_dimension)
		do
			page_dimension := a_page_dimension
		ensure
			page_dimension_set: page_dimension = a_page_dimension
		end

	set_title_page (a_boolean: BOOLEAN)
			-- Set `has_title_page' to `a_boolean'.
		do
			has_title_page := a_boolean
		ensure
			has_title_page_set: has_title_page = a_boolean
		end

	set_portrait (a_boolean: BOOLEAN)
			-- Set `is_portrait' to `a_boolean'.
		do
			is_portrait := a_boolean
		ensure
			is_portrait_set: is_portrait = a_boolean
		end

feature -- Basic operations

	output_document_collection (a_document_collection: EDL_DOCUMENT_COLLECTION; a_directory: DIRECTORY)
			-- Output `a_document_collection' into `a_directory' by translating
			-- all document to their output format representation and copying all
			-- images to the output directory.
		local
			output_directory: EDL_OUTPUT_DIRECTORY
			context: EDL_LATEX_TRANSLATOR_CONTEXT
			translator: EDL_LATEX_TRANSLATOR
			file_name: STRING
		do
			create output_directory.make (a_directory)
			create context.make (a_document_collection, Current)
			create translator.make_from_context (context)

			-- Translate all documents.
			from
				a_document_collection.start
			until
				a_document_collection.after
			loop
				translator.translate (a_document_collection.item)
				file_name := context.document_file_name (a_document_collection.item)

				output_directory.write_file (file_name, translator.last_result)
				a_document_collection.forth
			end

			-- Copy images to output directory.
			from
				context.images.start
			until
				context.images.after
			loop
				output_directory.copy_file (context.images.item.file, context.image_file_name (context.images.item))
				context.images.forth
			end
		end

feature {EDL_LATEX_TRANSLATOR} -- Implementation

	listing_language_mapping: HASH_TABLE [STRING, STRING]
			-- Mapping from cannoical names visible to the outside
			-- to internal names.
		once
			create Result.make(78)
			Result.put ("", "")
			Result.put ("ABAP", "ABAP")
			Result.put ("IDL", "IDL")
			Result.put ("Plasm", "Plasm")
			Result.put ("ACSL", "ACSL")
			Result.put ("inform", "inform")
			Result.put ("POV", "POV")
			Result.put ("Ada", "Ada")
			Result.put ("Java", "Java")
			Result.put ("Prolog", "Prolog")
			Result.put ("Algol", "Algol")
			Result.put ("JVMIS", "JVMIS")
			Result.put ("Promela", "Promela")
			Result.put ("Ant", "Ant")
			Result.put ("ksh", "ksh")
			Result.put ("Python", "Python")
			Result.put ("Assembler", "Assembler")
			Result.put ("Lisp", "Lisp")
			Result.put ("R", "R")
			Result.put ("Awk", "Awk")
			Result.put ("Logo", "Logo")
			Result.put ("Reduce", "Reduce")
			Result.put ("bash", "bash")
			Result.put ("make", "make")
			Result.put ("Rexx", "Rexx")
			Result.put ("Basic", "Basic")
			Result.put ("Mathematica", "Mathematica")
			Result.put ("RSL", "RSL")
			Result.put ("C", "C")
			Result.put ("Matlab", "Matlab")
			Result.put ("Ruby", "Ruby")
			Result.put ("C++", "C++")
			Result.put ("Mercury", "Mercury")
			Result.put ("S", "S")
			Result.put ("Caml", "Caml")
			Result.put ("MetaPost", "MetaPost")
			Result.put ("SAS", "SAS")
			Result.put ("Clean", "Clean")
			Result.put ("Miranda", "Miranda")
			Result.put ("Scilab", "Scilab")
			Result.put ("Cobol", "Cobol")
			Result.put ("Mizar", "Mizar")
			Result.put ("sh", "sh")
			Result.put ("Comal", "Comal")
			Result.put ("ML", "ML")
			Result.put ("SHELXL", "SHELXL")
			Result.put ("csh", "csh")
			Result.put ("Modula-2", "Modula-2")
			Result.put ("Simula", "Simula")
			Result.put ("Delphi", "Delphi")
			Result.put ("MuPAD", "MuPAD")
			Result.put ("SQL", "SQL")
			Result.put ("Eiffel", "Eiffel")
			Result.put ("NASTRAN", "NASTRAN")
			Result.put ("tcl", "tcl")
			Result.put ("Elan", "Elan")
			Result.put ("Oberon-2", "Oberon-2")
			Result.put ("TeX", "TeX")
			Result.put ("erlang", "erlang")
			Result.put ("OCL", "OCL")
			Result.put ("VBScript", "VBScript")
			Result.put ("Euphoria", "Euphoria")
			Result.put ("Octave", "Octave")
			Result.put ("Verilog", "Verilog")
			Result.put ("Fortran", "Fortran")
			Result.put ("Oz", "Oz")
			Result.put ("VHDL", "VHDL")
			Result.put ("GCL", "GCL")
			Result.put ("Pascal", "Pascal")
			Result.put ("VRML", "VRML")
			Result.put ("Gnuplot", "Gnuplot")
			Result.put ("Perl", "Perl")
			Result.put ("XML", "XML")
			Result.put ("Haskell", "Haskell")
			Result.put ("PHP", "PHP")
			Result.put ("XSLT", "XSLT")
			Result.put ("HTML", "HTML")
			Result.put ("PL/I", "PL/I")
		end

end
