note
	description: "[
		Representation of a color in the RGB color model (24 bit
		RGB representation).
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_COLOR

inherit
	HASHABLE

create
	make,
	make_rgb

feature {NONE} -- Initialization

	make (a_color: STRING)
			-- Initialize with a color string.
		require
			color_exists: names.has (a_color)
		local
			color: TUPLE [INTEGER, INTEGER, INTEGER]
		do
			color := names [a_color]

			red := color.integer_item (1)
			green := color.integer_item (2)
			blue := color.integer_item (3)
		ensure
			red_set: red = names [a_color].integer_item (1)
			green_set: green = names [a_color].integer_item (2)
			blue_set: blue = names [a_color].integer_item (3)
		end

	make_rgb (a_red, a_green, a_blue: INTEGER)
			-- Initialize with RGB color code.
		require
			valid_red: a_red >= 0 and a_red <= 255
			valid_green: a_green >= 0 and a_green <= 255
			valid_blue: a_blue >= 0 and a_blue <= 255
		do
			red := a_red
			green := a_green
			blue := a_blue
		ensure
			red_set: red = a_red
			green_set: green = a_green
			blue_set: blue = a_blue
		end

feature -- Access

	red: INTEGER
			-- Red color part.

	green: INTEGER
			-- Green color part.

	blue: INTEGER
			-- Blue color part.

	name: detachable STRING
			-- Name, if known.
		do
			from
				names.start
			until
				names.after or Result /= Void
			loop
				if names.item_for_iteration.is_equal ([red, green, blue]) then
					create Result.make_from_string (names.key_for_iteration)
				end

				names.forth
			end
		end

	hash_code: INTEGER
			-- Hash code value.
		do
			Result := red
			Result := Result + (green * 256)
			Result := Result + (blue * 65536)
		end

feature -- Miscellaneous

	names: HASH_TABLE [TUPLE [INTEGER, INTEGER, INTEGER], STRING]
			-- Predefined list of colors (names are all lower-case)
			-- The colors are based on the SVG color keywords of the
			-- CSS3 specification.
			-- (see http://www.w3.org/TR/css3-color/#svg-color)
		once
			create Result.make (10)
			Result.put ([240, 248, 255], "aliceblue")
			Result.put ([250, 235, 215], "antiquewhite")
			Result.put ([0, 255, 255], "aqua")
			Result.put ([127, 255, 212], "aquamarine")
			Result.put ([240, 255, 255], "azure")
			Result.put ([245, 245, 220], "beige")
			Result.put ([255, 228, 196], "bisque")
			Result.put ([0, 0, 0], "black")
			Result.put ([255, 235, 205], "blanchedalmond")
			Result.put ([0, 0, 255], "blue")
			Result.put ([138, 43, 226], "blueviolet")
			Result.put ([165, 42, 42], "brown")
			Result.put ([222, 184, 135], "burlywood")
			Result.put ([95, 158, 160], "cadetblue")
			Result.put ([127, 255, 0], "chartreuse")
			Result.put ([210, 105, 30], "chocolate")
			Result.put ([255, 127, 80], "coral")
			Result.put ([100, 149, 237], "cornflowerblue")
			Result.put ([255, 248, 220], "cornsilk")
			Result.put ([220, 20, 60], "crimson")
			Result.put ([0, 255, 255], "cyan")
			Result.put ([0, 0, 139], "darkblue")
			Result.put ([0, 139, 139], "darkcyan")
			Result.put ([184, 134, 11], "darkgoldenrod")
			Result.put ([169, 169, 169], "darkgray")
			Result.put ([0, 100, 0], "darkgreen")
			Result.put ([169, 169, 169], "darkgrey")
			Result.put ([189, 183, 107], "darkkhaki")
			Result.put ([139, 0, 139], "darkmagenta")
			Result.put ([85, 107, 47], "darkolivegreen")
			Result.put ([255, 140, 0], "darkorange")
			Result.put ([153, 50, 204], "darkorchid")
			Result.put ([139, 0, 0], "darkred")
			Result.put ([233, 150, 122], "darksalmon")
			Result.put ([143, 188, 143], "darkseagreen")
			Result.put ([72, 61, 139], "darkslateblue")
			Result.put ([47, 79, 79], "darkslategray")
			Result.put ([47, 79, 79], "darkslategrey")
			Result.put ([0, 206, 209], "darkturquoise")
			Result.put ([148, 0, 211], "darkviolet")
			Result.put ([255, 20, 147], "deeppink")
			Result.put ([0, 191, 255], "deepskyblue")
			Result.put ([105, 105, 105], "dimgray")
			Result.put ([105, 105, 105], "dimgrey")
			Result.put ([30, 144, 255], "dodgerblue")
			Result.put ([178, 34, 34], "firebrick")
			Result.put ([255, 250, 240], "floralwhite")
			Result.put ([34, 139, 34], "forestgreen")
			Result.put ([255, 0, 255], "fuchsia")
			Result.put ([220, 220, 220], "gainsboro")
			Result.put ([248, 248, 255], "ghostwhite")
			Result.put ([255, 215, 0], "gold")
			Result.put ([218, 165, 32], "goldenrod")
			Result.put ([128, 128, 128], "gray")
			Result.put ([0, 128, 0], "green")
			Result.put ([173, 255, 47], "greenyellow")
			Result.put ([128, 128, 128], "grey")
			Result.put ([240, 255, 240], "honeydew")
			Result.put ([255, 105, 180], "hotpink")
			Result.put ([205, 92, 92], "indianred")
			Result.put ([75, 0, 130], "indigo")
			Result.put ([255, 255, 240], "ivory")
			Result.put ([240, 230, 140], "khaki")
			Result.put ([230, 230, 250], "lavender")
			Result.put ([255, 240, 245], "lavenderblush")
			Result.put ([124, 252, 0], "lawngreen")
			Result.put ([255, 250, 205], "lemonchiffon")
			Result.put ([173, 216, 230], "lightblue")
			Result.put ([240, 128, 128], "lightcoral")
			Result.put ([224, 255, 255], "lightcyan")
			Result.put ([250, 250, 210], "lightgoldenrodyellow")
			Result.put ([211, 211, 211], "lightgray")
			Result.put ([144, 238, 144], "lightgreen")
			Result.put ([211, 211, 211], "lightgrey")
			Result.put ([255, 182, 193], "lightpink")
			Result.put ([255, 160, 122], "lightsalmon")
			Result.put ([32, 178, 170], "lightseagreen")
			Result.put ([135, 206, 250], "lightskyblue")
			Result.put ([119, 136, 153], "lightslategray")
			Result.put ([119, 136, 153], "lightslategrey")
			Result.put ([176, 196, 222], "lightsteelblue")
			Result.put ([255, 255, 224], "lightyellow")
			Result.put ([0, 255, 0], "lime")
			Result.put ([50, 205, 50], "limegreen")
			Result.put ([250, 240, 230], "linen")
			Result.put ([255, 0, 255], "magenta")
			Result.put ([128, 0, 0], "maroon")
			Result.put ([102, 205, 170], "mediumaquamarine")
			Result.put ([0, 0, 205], "mediumblue")
			Result.put ([186, 85, 211], "mediumorchid")
			Result.put ([147, 112, 219], "mediumpurple")
			Result.put ([60, 179, 113], "mediumseagreen")
			Result.put ([123, 104, 238], "mediumslateblue")
			Result.put ([0, 250, 154], "mediumspringgreen")
			Result.put ([72, 209, 204], "mediumturquoise")
			Result.put ([199, 21, 133], "mediumvioletred")
			Result.put ([25, 25, 112], "midnightblue")
			Result.put ([245, 255, 250], "mintcream")
			Result.put ([255, 228, 225], "mistyrose")
			Result.put ([255, 228, 181], "moccasin")
			Result.put ([255, 222, 173], "navajowhite")
			Result.put ([0, 0, 128], "navy")
			Result.put ([253, 245, 230], "oldlace")
			Result.put ([128, 128, 0], "olive")
			Result.put ([107, 142, 35], "olivedrab")
			Result.put ([255, 165, 0], "orange")
			Result.put ([255, 69, 0], "orangered")
			Result.put ([218, 112, 214], "orchid")
			Result.put ([238, 232, 170], "palegoldenrod")
			Result.put ([152, 251, 152], "palegreen")
			Result.put ([175, 238, 238], "paleturquoise")
			Result.put ([219, 112, 147], "palevioletred")
			Result.put ([255, 239, 213], "papayawhip")
			Result.put ([255, 218, 185], "peachpuff")
			Result.put ([205, 133, 63], "peru")
			Result.put ([255, 192, 203], "pink")
			Result.put ([221, 160, 221], "plum")
			Result.put ([176, 224, 230], "powderblue")
			Result.put ([128, 0, 128], "purple")
			Result.put ([255, 0, 0], "red")
			Result.put ([188, 143, 143], "rosybrown")
			Result.put ([65, 105, 225], "royalblue")
			Result.put ([139, 69, 19], "saddlebrown")
			Result.put ([250, 128, 114], "salmon")
			Result.put ([244, 164, 96], "sandybrown")
			Result.put ([46, 139, 87], "seagreen")
			Result.put ([255, 245, 238], "seashell")
			Result.put ([160, 82, 45], "sienna")
			Result.put ([192, 192, 192], "silver")
			Result.put ([135, 206, 235], "skyblue")
			Result.put ([106, 90, 205], "slateblue")
			Result.put ([112, 128, 144], "slategray")
			Result.put ([112, 128, 144], "slategrey")
			Result.put ([255, 250, 250], "snow")
			Result.put ([0, 255, 127], "springgreen")
			Result.put ([70, 130, 180], "steelblue")
			Result.put ([210, 180, 140], "tan")
			Result.put ([0, 128, 128], "teal")
			Result.put ([216, 191, 216], "thistle")
			Result.put ([255, 99, 71], "tomato")
			Result.put ([64, 224, 208], "turquoise")
			Result.put ([238, 130, 238], "violet")
			Result.put ([245, 222, 179], "wheat")
			Result.put ([255, 255, 255], "white")
			Result.put ([245, 245, 245], "whitesmoke")
			Result.put ([255, 255, 0], "yellow")
			Result.put ([154, 205, 50], "yellowgreen")
		end

invariant

	valid_red: red >= 0 and red <= 255
	valid_green: green >= 0 and green <= 255
	valid_blue: blue >= 0 and blue <= 255

end
