note
	description: "[
		Text elements contain the actual user-defined text and thus hold the actual content
		of a document. They also support some basic formatting options such as italics, bold
		face and underlining.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_TEXT_ELEMENT

inherit
	EDL_INLINE_ELEMENT

create
	make,
	make_italic,
	make_bold,
	make_underlined,
	make_superscript,
	make_subscript,
	make_with_color,
	make_with_background_color

convert
	make ({STRING}),
	text: {STRING}

feature {NONE} -- Initialization

	make (a_text: STRING)
			-- Initialize with `a_text'.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			text := a_text
		ensure
			text_set: text = a_text
			no_formatting: not is_italic and not is_bold and not is_underlined
			no_color: color = Void
			no_background_color: background_color = Void
		end

	make_italic (a_text: STRING)
			-- Initialize with italic `a_text'.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			make (a_text)
			is_italic := True
		ensure
			text_set: text = a_text
			is_only_italic: is_italic and not is_bold and not is_underlined
			no_color: color = Void
			no_background_color: background_color = Void
		end

	make_bold (a_text: STRING)
			-- Initialize with bold `a_text'.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			make (a_text)
			is_bold := True
		ensure
			text_set: text = a_text
			is_only_bold: not is_italic and is_bold and not is_underlined
			no_color: color = Void
			no_background_color: background_color = Void
		end

	make_underlined (a_text: STRING)
			-- Initialize with underlined `a_text'.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			make (a_text)
			is_underlined := True
		ensure
			text_set: text = a_text
			is_only_underlined: not is_italic and not is_bold and is_underlined
			no_color: color = Void
			no_background_color: background_color = Void
		end

	make_superscript (a_text: STRING)
			-- Initialize with `a_text' as superscript.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			make (a_text)
			is_superscript := True
		ensure
			text_set: text = a_text
			no_formatting: not is_italic and not is_bold and not is_underlined
			no_color: color = Void
			no_background_color: background_color = Void
			is_superscript_set: is_superscript
		end

	make_subscript (a_text: STRING)
			-- Initialize with `a_text' as subscript.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			make (a_text)
			is_subscript := True
		ensure
			text_set: text = a_text
			no_formatting: not is_italic and not is_bold and not is_underlined
			no_color: color = Void
			no_background_color: background_color = Void
			is_subscript_set: is_subscript
		end

	make_with_color (a_text: STRING; a_color: detachable EDL_COLOR)
			-- Initialize with `a_text' colored using `a_color'.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			make (a_text)
			color := a_color
		ensure
			text_set: text = a_text
			color_set: color = a_color
			no_formatting: not is_italic and not is_bold and not is_underlined
			no_background_color: background_color = Void
		end

	make_with_background_color (a_text: STRING; a_color: detachable EDL_COLOR)
			-- Initialize with `a_text' with the background color `a_color'.
		require
			text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_text)
		do
			make (a_text)
			background_color := a_color
		ensure
			text_set: text = a_text
			background_color_set: background_color = a_color
			no_formatting: not is_italic and not is_bold and not is_underlined
			no_color: color = Void
		end

feature -- Access

	text: STRING
			-- String contained in the element.

	color: detachable EDL_COLOR assign set_color
			-- Color of the text.

	background_color: detachable EDL_COLOR assign set_background_color
			-- Background color of the text.

	is_italic: BOOLEAN assign set_italic
			-- Is the element italic?

	is_bold: BOOLEAN assign set_bold
			-- Is the element bold?

	is_underlined: BOOLEAN assign set_underlined
			-- Is the element underlined?

	is_superscript: BOOLEAN assign set_superscript
			-- Is the element superscript?

	is_subscript: BOOLEAN assign set_subscript
			-- Is the element subscript?

feature -- Element change

	set_italic (a_boolean: BOOLEAN)
			-- Change `is_italic' to `a_boolean'.
		do
			is_italic := a_boolean
		ensure
			is_italic_set: is_italic = a_boolean
		end

	set_bold (a_boolean: BOOLEAN)
			-- Change `is_bold' to `a_boolean'.
		do
			is_bold := a_boolean
		ensure
			is_bold_set: is_bold = a_boolean
		end

	set_underlined (a_boolean: BOOLEAN)
			-- Change `is_underlined' to `a_boolean'.
		do
			is_underlined := a_boolean
		ensure
			is_underlined_set: is_underlined = a_boolean
		end

	set_color (a_color: detachable EDL_COLOR)
			-- Change `color' to `a_color'. If `a_color' is Void,
			-- the color is removed.
		do
			color := a_color
		ensure
			color_set: color = a_color
		end

	set_background_color (a_color: detachable EDL_COLOR)
			-- Change `background_color' to `a_color'. If `a_color' is Void,
			-- the background color is removed.
		do
			background_color := a_color
		ensure
			background_color_set: background_color = a_color
		end

	set_superscript (a_boolean: BOOLEAN)
			-- Change `is_superscript' to `a_boolean'.
		require
			not_subscript: not is_subscript
		do
			is_superscript := a_boolean
		ensure
			is_superscript_set: is_superscript = a_boolean
		end

	set_subscript (a_boolean: BOOLEAN)
			-- Change `is_subscript' to `a_boolean'.
		require
			not_superscript: not is_superscript
		do
			is_subscript := a_boolean
		ensure
			is_subscript_set: is_subscript = a_boolean
		end

feature -- Miscellaneous

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor'.
		do
			a_visitor.visit_text_element (Current)
			a_visitor.leave_text_element (Current)
		end

invariant

	not_subscript_and_superscript: not (is_subscript and is_superscript)
	text_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (text)

end
