note
	description: "[
		Facilities to handle UTF-8 encoded strings, e.g. to check
		whether a string is valid UTF-8 or converting a valid UTF-8
		string to its ASCII representation.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_UTF8_ENCODING

feature -- Basic operations

	is_valid (a_string: STRING): BOOLEAN
			-- Is `a_string' a valid UTF-8 encoded string.
		local
			i, char_length: INTEGER
			char_found: BOOLEAN
		do
			from
				Result := True
				i := 1
			until
				i > a_string.count or not Result
			loop
				from
					char_length := 1
					char_found := False
				until
					char_found or char_length > 4 or i + char_length - 1 > a_string.count
				loop
					if attached a_string.substring (i, i + char_length - 1) as s and then is_valid_character (s) then
						char_found := True
					else
						char_length := char_length + 1
					end
				end

				if not char_found then
					Result := False
				end

				i := i + char_length
			end
		end

	to_ascii (a_string: STRING): STRING
			-- Convert a valid UTF-8 encoded string to ASCII, dropping all non-ASCII characters.
		require
			is_valid: is_valid (a_string)
		local
			i: INTEGER
			char_length: INTEGER
		do
			create Result.make_empty

			from
				i := 1
			until
				i > a_string.count
			loop
				from
					char_length := 1
				until
					attached a_string.substring (i, i + char_length - 1) as substring and then is_valid_character (substring)
				loop
					char_length := char_length + 1
				end

				if char_length = 1 then
					Result.append (a_string.substring (i, i + char_length - 1))
				end

				i := i + char_length
			end
		end

feature {NONE} -- Implementation

	is_valid_character (a_char: STRING): BOOLEAN
			-- Is `a_char' a valid UTF-8 encoded character (represented using 1 to 4 bytes).
		do
			inspect
				a_char.count
			when 1 then
				Result := a_char.at (1).code = 0x09 or a_char.at (1).code = 0x0A or a_char.at (1).code = 0x20
				Result := Result or (a_char.at (1).code >= 0x20 and a_char.at (1).code <= 0x7E)
			when 2 then
				Result := (a_char.at (1).code >= 0xC2 and a_char.at (1).code <= 0xDF)
					and (a_char.at (2).code >= 0x80 and a_char.at (2).code <= 0xBF)
			when 3 then
				inspect
					a_char.at (1).code
				when 0xE0 then
					Result := (a_char.at (2).code >= 0xA0 and a_char.at (2).code <= 0xBF)
						and (a_char.at (3).code >= 0x80 and a_char.at (3).code <= 0xBF)
				when 0xED then
					Result := (a_char.at (2).code >= 0x80 and a_char.at (2).code <= 0x9F)
						and (a_char.at (3).code >= 0x80 and a_char.at (3).code <= 0xBF)
				else
					Result := ((a_char.at (1).code >= 0xE1 and a_char.at (1).code <= 0xEC)
						or (a_char.at (1).code >= 0xEE and a_char.at (1).code <= 0xEF))
						and (a_char.at (2).code >= 0x80 and a_char.at (2).code <= 0xBF)
						and (a_char.at (3).code >= 0x80 and a_char.at (3).code <= 0xBF)
				end
			when 4 then
				inspect
					a_char.at (1).code
				when 0xF0 then
					Result := (a_char.at (2).code >= 0x90 and a_char.at (2).code <= 0xBF)
						and (a_char.at (3).code >= 0x80 and a_char.at (3).code <= 0xBF)
						and (a_char.at (4).code >= 0x80 and a_char.at (4).code <= 0xBF)
				when 0xF4 then
					Result := (a_char.at (2).code >= 0x80 and a_char.at (2).code <= 0x8F)
						and (a_char.at (3).code >= 0x80 and a_char.at (3).code <= 0xBF)
						and (a_char.at (4).code >= 0x80 and a_char.at (4).code <= 0xBF)
				else
					Result := (a_char.at (1).code >= 0xF1 and a_char.at (1).code <= 0xF3)
						and (a_char.at (2).code >= 0x80 and a_char.at (2).code <= 0xBF)
						and (a_char.at (3).code >= 0x80 and a_char.at (3).code <= 0xBF)
						and (a_char.at (4).code >= 0x80 and a_char.at (4).code <= 0xBF)
				end
			else
				Result := False -- more than 4 bytes are not valid
			end
		end

end
