note
	description: "[
		Facilities that output documents or document collections to a directory.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_WRITER

feature -- Measurement

	document_fulfills_all_constraints (a_document: EDL_DOCUMENT): BOOLEAN
			-- Does `a_document' fulfill all requirements
			-- specified by this writer/output format?
		local
			c: EDL_ELEMENT_COLLECTOR [EDL_LISTING]
		do
			-- List item constraints
			Result := a_document.fulfills_constraint (create {EDL_CHILDREN_COUNT_CONSTRAINT [EDL_ROMAN_LIST]}.make_maximum (maximum_roman_number))
			Result := Result and a_document.fulfills_constraint (create {EDL_CHILDREN_COUNT_CONSTRAINT [EDL_ALPHABETIC_LIST]}.make_maximum (letters_in_alphabet))

			-- No empty lists, tables, table rows or references
			Result := Result and a_document.fulfills_constraint (create {EDL_CHILDREN_COUNT_CONSTRAINT [EDL_LIST]}.make_minimum (1))
			Result := Result and a_document.fulfills_constraint (create {EDL_CHILDREN_COUNT_CONSTRAINT [EDL_TABLE]}.make_minimum (1))
			Result := Result and a_document.fulfills_constraint (create {EDL_CHILDREN_COUNT_CONSTRAINT [EDL_TABLE_ROW]}.make_minimum (1))
			Result := Result and a_document.fulfills_constraint (create {EDL_CHILDREN_COUNT_CONSTRAINT [EDL_REFERENCE]}.make_minimum (1))

			-- Only listings with supported languages
			create c
			c.collect (a_document)
			Result := Result and c.last_list.for_all (agent (l: EDL_LISTING): BOOLEAN do
					Result := supports_listing_language (l.language)
				end)
		end

	document_has_no_invalid_references (a_document: EDL_DOCUMENT): BOOLEAN
			-- Are all references in `a_document' valid? A reference is valid if
			--  - it is a reference to an URI
			--  - it is a reference to a document and points to `a_document'
			--  - it is a reference to a section and this section is part of
			--    `a_document'.
		local
			document_collection: EDL_DOCUMENT_COLLECTION
		do
			create document_collection.make
			document_collection.extend (a_document)
			Result := document_collection_has_no_invalid_references (document_collection)
		end

	document_collection_has_no_invalid_references (a_document_collection: EDL_DOCUMENT_COLLECTION): BOOLEAN
			-- Are all references in `a_document_collection' valid? A reference is valid if
			--  - it is a reference to an URI
			--  - it is a reference to a document and points to a document contained in
			--    `a_document_collection'.
			--  - it is a reference to a section and this section is part of
			--    a document that is part of `a_document_collection'.
		local
			reference_collector: EDL_ELEMENT_COLLECTOR [EDL_REFERENCE]
			reference_list: LINKED_LIST [EDL_REFERENCE]
			destination: detachable EDL_ELEMENT
		do
			Result := True

			from
				create reference_collector
				a_document_collection.start
			until
				a_document_collection.after or not Result
			loop
				reference_collector.collect (a_document_collection.item)
				reference_list := reference_collector.last_list

				from
					reference_list.start
				until
					reference_list.after or not Result
				loop
					if attached reference_list.item.destination_document as document then
						destination := document
					elseif attached reference_list.item.destination_section as section then
						destination := section
					end

					if attached destination as attached_destination then
						if attached destination.parent_document as document then
							Result := a_document_collection.has (document)
						else
							Result := False
						end
					end

					reference_list.forth
				end

				a_document_collection.forth
			end
		end

	maximum_roman_number: INTEGER = 3888
			-- Number of simple roman numbers.

	letters_in_alphabet: INTEGER = 26
	 		-- Number of letters in the normal alphabet.

	supports_listing_language (a_language: STRING): BOOLEAN
			-- Is `a_language' supported for listing elements?
		deferred
		end

feature -- Basic operations

	output_document_collection (a_document_collection: EDL_DOCUMENT_COLLECTION; a_directory: DIRECTORY)
			-- Output all documents in `a_document_collection' to `a_directory'.
		require
			valid_directory: a_directory.exists implies (a_directory.is_empty and a_directory.is_writable)
			non_empty_collection: a_document_collection.count > 0
			all_documents_fulfill_all_constraints: a_document_collection.for_all (agent document_fulfills_all_constraints)
			valid_references: document_collection_has_no_invalid_references (a_document_collection)
		deferred
		ensure
			directory_exists: a_directory.exists
			directory_is_not_empty: not a_directory.is_empty
		end

	output_document (a_document: EDL_DOCUMENT; a_directory: DIRECTORY)
			-- Output `a_document' to `a_directory'.
		require
			valid_directory: a_directory.exists implies (a_directory.is_empty and a_directory.is_writable)
			valid_document: document_fulfills_all_constraints (a_document)
			valid_references: document_has_no_invalid_references (a_document)
		local
			collection: EDL_DOCUMENT_COLLECTION
		do
			create collection.make
			collection.extend (a_document)
			output_document_collection (collection, a_directory)
		ensure
			directory_exists: a_directory.exists
			directory_is_not_empty: not a_directory.is_empty
		end

end
