note
	description: "[
		Visitors that translate documents to HTML given a
		EDL_HTML_TRANSLATOR_CONTEXT.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_HTML_TRANSLATOR

inherit
	EDL_TEXT_TRANSLATOR
		redefine
			reset
		end

	EDL_SINK_VISITOR
		redefine
			visit_text_element,
			leave_text_element,
			visit_table,
			visit_table_row,
			visit_table_cell,
			visit_alphabetic_list,
			visit_arabic_list,
			visit_bullet_list,
			visit_roman_list,
			visit_list_item,
			visit_section,
			leave_section,
			visit_image,
			visit_paragraph,
			leave_paragraph,
			visit_reference,
			visit_table_of_contents,
			leave_table_of_contents,
			visit_raw_element,
			leave_raw_element,
			visit_listing,
			visit_document,
			leave_document
		end

create
	make_from_context

feature {NONE} -- Initialization

	make_from_context (a_context: EDL_HTML_TRANSLATOR_CONTEXT)
			-- Initialize with `a_context'.
		do
			reset

			context := a_context
		end

feature {EDL_VISITABLE} -- Visitor

	visit_element (a_element: EDL_ELEMENT)
			-- Do nothing.
		do
		end

	leave_element (a_element: EDL_ELEMENT)
			-- Append the end of the HTML representation of `a_element'
			-- to `last_result'.
		do
			close_tag
		end

	visit_text_element (a_text_element: EDL_TEXT_ELEMENT)
			-- Append the beginning of the HTML representation of `a_text_element'
			-- to `last_result'.
		local
			internal_tag: EDL_HTML_TAG
			temp_tag: EDL_HTML_TAG
			formatting: HASH_TABLE [BOOLEAN, STRING]
			style_mapping: HASH_TABLE [STRING, STRING]
		do
			internal_tag := current_tag

			create formatting.make (5)
			formatting.put (a_text_element.is_superscript, "sup")
			formatting.put (a_text_element.is_subscript, "sub")
			formatting.put (a_text_element.is_bold, "b")
			formatting.put (a_text_element.is_italic, "i")
			formatting.put (a_text_element.is_underlined, "u")

			from
				formatting.start
			until
				formatting.after
			loop
				if formatting.item_for_iteration then
					create temp_tag.make (formatting.key_for_iteration)
					internal_tag.extend (temp_tag)
					internal_tag := temp_tag
				end

				formatting.forth
			end

			create style_mapping.make (2)

			if attached a_text_element.color as color then
				style_mapping.put (css_color (color), "color")
			end

			if attached a_text_element.background_color as color then
				style_mapping.put (css_color (color), "background-color")
			end

			if not style_mapping.is_empty then
				temp_tag := "span"
				temp_tag.set_style (style_mapping)
				internal_tag.extend (temp_tag)
				internal_tag := temp_tag
			end

			internal_tag.extend_with_text (a_text_element.text)
		end

	leave_text_element (a_text_element: EDL_TEXT_ELEMENT)
			-- Do nothing.
		do
		end

	visit_table (a_table: EDL_TABLE)
			-- Append the beginning of the HTML representation of `a_table'
			-- to `last_result'.
		do
			open_simple_tag ("table")

			if a_table.borders_inside_horizontal then
				current_tag.add_class ("borders-inside-horizontal")
			end

			if a_table.borders_inside_vertical then
				current_tag.add_class ("borders-inside-vertical")
			end

			if a_table.borders_outside_horizontal then
				current_tag.add_class ("borders-outside-horizontal")
			end

			if a_table.borders_outside_vertical then
				current_tag.add_class ("borders-outside-vertical")
			end
		end

	visit_table_row (a_table_row: EDL_TABLE_ROW)
			-- Append the beginning of the HTML representation of `a_table_row'
			-- to `last_result'.
		do
			open_simple_tag ("tr")
		end

	visit_table_cell (a_table_cell: EDL_TABLE_CELL)
			-- Append the beginning of the HTML representation of `a_table_cell'
			-- to `last_result'.
		do
			if a_table_cell.is_heading then
				open_simple_tag ("th")
			else
				open_simple_tag ("td")
			end
		end

	visit_alphabetic_list (a_alphabetic_list: EDL_ALPHABETIC_LIST)
			-- Append the beginning of the HTML representation of `a_alphabetic_list'
			-- to `last_result'.
		local
			type: STRING
		do
			type := "a"

			if a_alphabetic_list.is_upper_case then
				type.to_upper
			end

			open_tag (create {EDL_HTML_TAG}.make_with_attribute ("ol", "type", type))
		end

	visit_arabic_list (a_arabic_list: EDL_ARABIC_LIST)
			-- Append the beginning of the HTML representation of `a_arabic_list'
			-- to `last_result'.
		do
			open_simple_tag ("ol")
		end

	visit_bullet_list (a_bullet_list: EDL_BULLET_LIST)
			-- Append the beginning of the HTML representation of `a_bullet_list'
			-- to `last_result'.
		do
			open_simple_tag ("ul")
		end

	visit_roman_list (a_roman_list: EDL_ROMAN_LIST)
			-- Append the beginning of the HTML representation of `a_roman_list'
			-- to `last_result'.
		local
			type: STRING
		do
			type := "i"

			if a_roman_list.is_upper_case then
				type.to_upper
			end

			open_tag (create {EDL_HTML_TAG}.make_with_attribute ("ol", "type", type))
		end

	visit_list_item (a_table: EDL_LIST_ITEM)
			-- Append the beginning of the HTML representation of `a_table'
			-- to `last_result'.
		do
			open_simple_tag ("li")
		end

	visit_section (a_section: EDL_SECTION)
			-- Append the beginning of the HTML representation of `a_section'
			-- to `last_result'.
		local
			tag: STRING
			ht: EDL_HTML_TAG
		do
			section_nesting_depth := section_nesting_depth + 1
			tag := "h"
			tag.append_integer (section_nesting_depth)

			create ht.make_with_text (tag, section_title_with_number (a_section))
			ht.attributes.put (context.section_id (a_section), "id")

			current_tag.extend (ht)
		end

	leave_section (a_section: EDL_SECTION)
			-- Append the end of the HTML representation of `a_section'
			-- to `last_result'.
		do
			section_nesting_depth := section_nesting_depth - 1
		end

	visit_image (a_image: EDL_IMAGE)
			-- Append the HTML representation of `a_image'
			-- to `last_result'.
		local
			img_tag: EDL_HTML_TAG
		do
			img_tag := "img"
			img_tag.attributes.put (context.image_file_name (a_image), "src")
			img_tag.attributes.put (a_image.name, "alt")
			img_tag.has_closing_tag := False

			current_tag.extend (img_tag)
		end

	visit_paragraph (a_paragraph: EDL_PARAGRAPH)
			-- Append the beginning of the HTML representation of `a_paragraph'
			-- to `last_result'.
		do
			if is_paragraph_tag_required (a_paragraph) then
				open_simple_tag ("p")
			end

			current_tag.is_inline := True
		end

	leave_paragraph (a_paragraph: EDL_PARAGRAPH)
			-- Append the end of the HTML representation of `a_paragraph'
			-- to `last_result'.
		do
			if is_paragraph_tag_required (a_paragraph) then
				close_tag
			end
		end

	visit_reference (a_reference: EDL_REFERENCE)
			-- Append the beginning of the HTML representation of `a_reference'
			-- to `last_result'.
		do
			open_tag (create {EDL_HTML_TAG}.make_with_attribute ("a", "href", context.resolve_reference (a_reference)))
		end

	visit_table_of_contents (a_table_of_contents: EDL_TABLE_OF_CONTENTS)
			-- Append the HTML representation of `a_table_of_contents'
			-- to `last_result'.
		local
			tag_name: STRING
			tag: EDL_HTML_TAG
		do
			if attached a_table_of_contents.title as t then
				tag_name := "h"
				tag_name.append_integer (section_nesting_depth + 1)

				create tag.make_with_text (tag_name, t)
				current_tag.extend (tag)
			end

			if section_forest.count > 0 then
				open_tag (create {EDL_HTML_TAG}.make_with_attribute ("ol", "class", "table-of-contents"))

				from
					section_forest.start
				until
					section_forest.after
				loop
					visit_table_of_contents_helper (section_forest.item_for_iteration)
					section_forest.forth
				end

				close_tag
			end
		end

	leave_table_of_contents (a_table_of_contents: EDL_TABLE_OF_CONTENTS)
			-- Do nothing.
		do
		end

	visit_raw_element (a_raw_element: EDL_RAW_ELEMENT)
			-- Append content of `a_raw_element' to `last_result'.
		local
			tn: EDL_HTML_TEXT_NODE
		do
			create tn.make_unescaped (a_raw_element.content)
			tn.is_undistorted := True
			current_tag.extend (tn)
		end

	leave_raw_element (a_raw_element: EDL_RAW_ELEMENT)
			-- Do nothing.
		do
		end

	visit_listing (a_listing: EDL_LISTING)
			-- Append the HTML representation of `a_listing' to `last_result'.
		local
			s: STRING
			lang: STRING
			tn: EDL_HTML_TEXT_NODE
			ht: EDL_HTML_TAG
		do
			lang := context.writer.listing_language_mapping.at (a_listing.language)
			listing_languages.extend (lang)
			s := "brush: "
			s.append (lang.as_lower)

			ht := "pre"
			ht.add_class (s)
			open_tag (ht)

			create tn.make (a_listing.content)
			tn.is_undistorted := True
			current_tag.extend (tn)
		end

	visit_document (a_document: EDL_DOCUMENT)
			-- Append the beginning of the HTML representation of `a_document'
			-- to `last_result'.
		local
			section_number_collector: EDL_SECTION_NUMBER_COLLECTOR
			ht: EDL_HTML_TAG
		do
			create section_number_collector

			section_number_collector.collect (a_document)
			section_numbers := section_number_collector.last_numbers
			section_forest := section_number_collector.last_forest

			last_result.append ("<!DOCTYPE HTML PUBLIC %"-//W3C//DTD HTML 4.01 Transitional//EN%"%N")
            last_result.append ("%T%"http://www.w3.org/TR/html4/loose.dtd%">%N")

			open_simple_tag ("html")
			open_simple_tag ("body")

			create ht.make_with_text ("h1", a_document.title)
			current_tag.extend (ht)

			section_nesting_depth := 1
		end

	leave_document (a_document: EDL_DOCUMENT)
			-- Append the end of the HTML representation of `a_document'
			-- to `last_result'.
		local
			head_tag: EDL_HTML_TAG
			temp_tag: EDL_HTML_TAG
			s: STRING
		do
			head_tag := "head"

			if listing_languages.count > 0 then
				temp_tag := "script"
				temp_tag.attributes.put ("text/javascript", "type")
				temp_tag.extend_with_unescaped_text ("SyntaxHighlighter.all()")
				current_tag.extend (temp_tag)
			end

			close_tag -- Close body tag.
			close_tag -- Close HTML tag.

			temp_tag := "title"
			temp_tag.extend_with_text (a_document.title)
			head_tag.extend (temp_tag)

			temp_tag := "meta"
			temp_tag.attributes.put ("Content-Type", "http-equiv")
			temp_tag.attributes.put ("text/html;charset=utf-8", "content")
			temp_tag.has_closing_tag := False
			head_tag.extend (temp_tag)

			-- Syntax highlighting
			if listing_languages.count > 0 then
				head_tag.extend (external_javascript_tag (syntax_highlighter_url ("scripts/shCore.js")))

				from
					listing_languages.start
				until
					listing_languages.after
				loop
					s := "scripts/shBrush"
					s.append (listing_languages.item)
					s.append (".js")
					head_tag.extend (external_javascript_tag (syntax_highlighter_url (s)))
					listing_languages.forth
				end

				head_tag.extend (external_css_tag (syntax_highlighter_url ("styles/shCore.css")))
				head_tag.extend (external_css_tag (syntax_highlighter_url ("styles/shThemeDefault.css")))
			end

			head_tag.extend (external_css_tag (context.css_file_name))

			if attached a_document.author as a then
				head_tag.extend (meta_tag ("author", a))
			end

			-- DT_DATE does not support formated output.
			if attached a_document.date as d then
				create s.make_empty
				s.append_integer (d.year)
				s.append ("-")
				if d.month < 10 then
					s.append ("0")
				end
				s.append_integer (d.month)
				s.append ("-")
				if d.day < 10 then
					s.append ("0")
				end
				s.append_integer (d.day)

				head_tag.extend (meta_tag ("date", s))
			end

			-- Produce actual output.
			check attached root_tag as html_tag then
				html_tag.put_front (head_tag)
				last_result.append (html_tag.out_indented (0))
			end
		end

feature {NONE} -- Implementation

	visit_table_of_contents_helper (a_section_tree: LINKED_TREE [EDL_SECTION])
			-- Append the partial table of contents representation of `a_section_tree'
			-- to `last_result'.
		local
			ref: EDL_REFERENCE
		do
			open_simple_tag ("li")

			create ref.make_simple_from_section (a_section_tree.item, section_title_with_number (a_section_tree.item))
			ref.accept (Current)

			if a_section_tree.arity > 0 then
				open_simple_tag ("ol")

				from
					a_section_tree.child_start
				until
					a_section_tree.child_after
				loop
					if attached a_section_tree.child as child_tree then
						visit_table_of_contents_helper (child_tree)
					end
					a_section_tree.child_forth
				end

				close_tag
			end

			close_tag
		end

	is_paragraph_tag_required (a_paragraph: EDL_PARAGRAPH): BOOLEAN
			-- Is the paragraph tag (<p>) required for `a_paragraph'?
		do
			if not attached {EDL_TABLE_CELL} a_paragraph.parent and not attached {EDL_LIST_ITEM} a_paragraph.parent then
				Result := True
			elseif attached {EDL_COMPOUND_ELEMENT [EDL_ELEMENT]} a_paragraph.parent as p then
				if p.count > 1 then
					Result := True
				end
			end
		end

	section_title_with_number (a_section: EDL_SECTION): STRING
			-- Number and title of `a_section' separated by a space.
		require
			known_section: section_numbers.has (a_section)
		do
			create Result.make_empty

			Result.append (section_numbers [a_section])
			Result.append_character (' ')
			Result.append (a_section.title)
		end

	css_color (a_color: EDL_COLOR): STRING
			-- Representation of `a_color' to be used in a CSS file.
		do
			create Result.make_empty

			if attached a_color.name as name and then supported_css_color_names.has (name) then
				Result := name
			else
				Result.append ("rgb(")
				Result.append (a_color.red.out)
				Result.append (", ")
				Result.append (a_color.green.out)
				Result.append (", ")
				Result.append (a_color.blue.out)
				Result.append (")")
			end
		end

	supported_css_color_names: LIST [STRING]
			-- List of color names defined in the CSS 2.1 Specification.
			-- See http://www.w3.org/TR/CSS2/syndata.html#color-units
		once
			create {LINKED_LIST [STRING]} Result.make
			Result.compare_objects

			Result.extend ("aqua")
			Result.extend ("black")
			Result.extend ("blue")
			Result.extend ("fuchsia")
			Result.extend ("gray")
			Result.extend ("green")
			Result.extend ("lime")
			Result.extend ("maroon")
			Result.extend ("navy")
			Result.extend ("olive")
			Result.extend ("orange")
			Result.extend ("purple")
			Result.extend ("red")
			Result.extend ("silver")
			Result.extend ("teal")
			Result.extend ("white")
			Result.extend ("yellow")
		end

	external_css_tag (a_url: STRING): EDL_HTML_TAG
			-- HTML <script> tag referring to JavaScript file `a_url'.
		require
			non_empty_url: not a_url.is_empty
		do
			Result := "link"
			Result.attributes.put (a_url, "href")
			Result.attributes.put ("stylesheet", "rel")
			Result.attributes.put ("text/css", "type")
			Result.has_closing_tag := False
		end

	external_javascript_tag (a_url: STRING): EDL_HTML_TAG
			-- HTML <link> tag referring to CSS file `a_url'.
		require
			non_empty_url: not a_url.is_empty
		do
			Result := "script"
			Result.attributes.put (a_url, "src")
			Result.attributes.put ("text/javascript", "type")
		end

	meta_tag (a_name: STRING; a_content: STRING): EDL_HTML_TAG
			-- HTML <meta> tag with name `a_name' and content `a_content'.
		require
			non_empty_name: not a_name.is_empty
		do
			Result := "meta"
			Result.attributes.put (a_name, "name")
			Result.attributes.put (a_content, "content")
			Result.has_closing_tag := False
		end

	syntax_highlighter_url (a_path: STRING): STRING
			-- URL to SyntaxHighlighter source file, given a relative `a_path'.
		require
			script_or_style_file: a_path.starts_with ("scripts/") or a_path.starts_with ("styles/")
		do
			Result := "http://alexgorbatchev.com/pub/sh/current/"
			Result.append (a_path)
		ensure
			correct_relative_path: Result.ends_with (a_path)
		end

	section_numbers: HASH_TABLE [STRING, EDL_SECTION]
			-- Mapping from sections to section numbers.

	section_forest: LINKED_LIST [LINKED_TREE [EDL_SECTION]]
			-- List of trees containing all sections, representing
			-- the section nesting of the current document.

	section_nesting_depth: INTEGER
			-- Current section nesting depth.

	listing_languages: LINKED_SET [STRING]
			-- Which languages are used in listing elements of the current document?

	context: EDL_HTML_TRANSLATOR_CONTEXT
			-- Current context.

	open_tag (a_tag: EDL_HTML_TAG)
			-- Open `a_tag'.
		do
			if tag_stack.is_empty then
				root_tag := a_tag
			else
				current_tag.extend (a_tag)
			end

			tag_stack.extend (a_tag)
		ensure
			correct_tag_stack_count: tag_stack.count = old tag_stack.count + 1
			tag_set: current_tag = a_tag
		end

	open_simple_tag (a_tag: STRING)
			-- Open an HTML tag called `a_tag'.
		require
			non_empty_tag: not a_tag.is_empty
		do
			open_tag (create {EDL_HTML_TAG}.make (a_tag))
		ensure
			correct_tag_stack_count: tag_stack.count = old tag_stack.count + 1
			tag_set: current_tag.tag.is_equal (a_tag)
			no_attributes: current_tag.attributes.count = 0
			not_inline: not current_tag.is_inline
		end

	close_tag
			-- Close `current_tag'.
		require
			non_empty_tag_stack: not tag_stack.is_empty
		do
			tag_stack.remove
		ensure
			correct_tag_stack_count: tag_stack.count = old tag_stack.count - 1
		end

	root_tag: detachable EDL_HTML_TAG
			-- Root tag of the current output.

	current_tag: EDL_HTML_TAG
			-- Currently active tag.
		require
			non_empty_tag_stack: not tag_stack.is_empty
		do
			Result := tag_stack.item
		ensure
			correct_current_tag: Result = tag_stack.item
		end

	tag_stack: LINKED_STACK [EDL_HTML_TAG]
			-- Stack of all open tags.

	reset
			-- Reset internal state.
		do
			Precursor

			create section_numbers.make (0)
			create section_forest.make
			create listing_languages.make
			create tag_stack.make
		ensure then
			empty_section_numbers: section_numbers.is_empty
			empty_section_forest: section_forest.is_empty
			empty_listing_languages: listing_languages.is_empty
			empty_tag_stack: tag_stack.is_empty
		end

invariant

	correct_current_tag: not tag_stack.is_empty implies current_tag = tag_stack.item

end
