note
	description: "[
		Facilities that output documents or document collections to a directory
		as HTML files, alongside potential user-defined images and a CSS file.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_HTML_WRITER

inherit
	EDL_WRITER
		redefine
			document_fulfills_all_constraints,
			output_document_collection,
			supports_listing_language,
			default_create
		end

create
	default_create,
	make_from_css_file

feature {NONE} -- Initialization

	default_create
			-- Initialize with default CSS file.
		do
			make_from_css_file (default_css_file)
		end

	make_from_css_file (a_css_file: FILE)
			-- Initialize with CSS file `a_css_file'.
		require
			css_file_exists: a_css_file.exists
			css_file_is_readable: a_css_file.is_readable
			has_css_file_extension: attached a_css_file.name as name and then name.ends_with (css_file_extension)
		do
			css_file := a_css_file
		ensure
			css_file_set: css_file = a_css_file
		end

feature -- Measurement

	document_fulfills_all_constraints (a_document: EDL_DOCUMENT): BOOLEAN
			-- Does `a_document' fulfill all requirements of the HTML writer?	
		do
			Result := Precursor (a_document)

			-- Valid section nesting
			Result := Result and a_document.fulfills_constraint (create {EDL_NESTING_DEPTH_CONSTRAINT [EDL_SECTION]}.make (maximum_section_nesting))
		end

	maximum_section_nesting: INTEGER = 5
			-- Maximum section nesting supported by HTML.

	css_file_extension: STRING = ".css"
			-- File extension for CSS files.

	supports_listing_language (a_language: STRING): BOOLEAN
			-- Is `a_language' supported for listing elements?
		do
			Result := listing_language_mapping.has (a_language)
		end

feature -- Basic operations

	output_document_collection (a_document_collection: EDL_DOCUMENT_COLLECTION; a_directory: DIRECTORY)
			-- Output `a_document_collection' into `a_directory' by translating
			-- all document to their output format representation and copying all
			-- images and the CSS file to the output directory.
		local
			output_directory: EDL_OUTPUT_DIRECTORY
			context: EDL_HTML_TRANSLATOR_CONTEXT
			translator: EDL_HTML_TRANSLATOR
			file_name: STRING
		do
			create output_directory.make (a_directory)
			create context.make (a_document_collection, Current)
			create translator.make_from_context (context)

			-- Translate all documents.
			from
				a_document_collection.start
			until
				a_document_collection.after
			loop
				translator.translate (a_document_collection.item)
				file_name := context.document_file_name (a_document_collection.item)

				output_directory.write_file (file_name, translator.last_result)
				a_document_collection.forth
			end

			-- Copy CSS file to output direcotry.
			output_directory.copy_file (css_file, context.css_file_name)

			-- Copy images to output directory.
			from
				context.images.start
			until
				context.images.after
			loop
				output_directory.copy_file (context.images.item.file, context.image_file_name (context.images.item))
				context.images.forth
			end
		end

feature -- Access

	css_file: FILE
			-- The CSS file used to specify presentation semantics of
			-- the final ouput.

feature -- Element change

	set_css_file (a_css_file: FILE)
			-- Set the CSS to `a_css_file'.
		require
			css_file_exists: a_css_file.exists
			css_file_is_readable: a_css_file.is_readable
			has_css_file_extension: attached a_css_file.name as name and then name.ends_with (css_file_extension)
		do
			css_file := a_css_file
		ensure
			css_file_set: css_file = a_css_file
		end

feature {EDL_HTML_TRANSLATOR} -- Implementation

	listing_language_mapping: HASH_TABLE [STRING, STRING]
			-- Mapping from cannoical names visible to the outside
			-- to internal names.
		once
			create Result.make(23)
			Result.put ("AS3", "ActionScript3")
			Result.put ("Bash", "Bash/shell")
			Result.put ("ColdFusion", "ColdFusion")
			Result.put ("CSharp", "C#")
			Result.put ("Cpp", "C++")
			Result.put ("Css", "CSS")
			Result.put ("Delphi", "Delphi")
			Result.put ("Diff", "Diff")
			Result.put ("Erlang", "Erlang")
			Result.put ("Groovy", "Groovy")
			Result.put ("JScript", "JavaScript")
			Result.put ("Java", "Java")
			Result.put ("JavaFX", "JavaFX")
			Result.put ("Perl", "Perl")
			Result.put ("Php", "PHP")
			Result.put ("Plain", "Plain Text")
			Result.put ("PowerShell", "PowerShell")
			Result.put ("Python", "Python")
			Result.put ("Ruby", "Ruby")
			Result.put ("Scala", "Scala")
			Result.put ("Sql", "SQL")
			Result.put ("Vb", "Visual Basic")
			Result.put ("Xml", "XML")
		end

feature {NONE} -- Implementation

	default_css_file: RAW_FILE
			-- Default CSS file, if the user does not provide an own file.
		local
			l_temporary_file_name: FILE_NAME
		once
			create l_temporary_file_name.make_temporary_name
			l_temporary_file_name.add_extension (css_file_extension)

			create Result.make_create_read_write (l_temporary_file_name)
			Result.put_string (default_css)
			Result.flush
			Result.close
		end

	default_css: STRING
			-- Default CSS to specify some basic presentation semantics of the
			-- final output.
		once
			create Result.make_empty

			Result.append ("body {%N")
			Result.append ("%Tfont-family: sans-serif;%N")
			Result.append ("%Tmargin: 0;%N")
			Result.append ("%Tpadding: 0 10px 0;%N")
			Result.append ("}%N%N")

			Result.append ("a {%N")
			Result.append ("%Tcolor: #1F497D;%N")
			Result.append ("%Ttext-decoration: none;%N")
			Result.append ("}%N%N")

			Result.append ("a:hover {%N")
			Result.append ("%Ttext-decoration: underline;%N")
			Result.append ("}%N%N")

			Result.append ("p, li {%N")
			Result.append ("%Ttext-align: justify;%N")
			Result.append ("%Tline-height: 130%%;%N")
			Result.append ("}%N%N")

			Result.append ("h1, h2, h3, h4, h5, h6 {%N")
			Result.append ("%Tfont-family: sans-serif;%N")
			Result.append ("%Tfont-weight: bold;%N")
			Result.append ("%Tbackground-color: #ECF2FE;%N")
			Result.append ("%Tcolor: #274A72;%N")
			Result.append ("%Tborder-top: 1px solid #ADC6EA;%N")
			Result.append ("%Tborder-bottom: 1px solid #ADC6EA;%N")
			Result.append ("%Tmargin: 10px -20px 10px -10px;%N")
			Result.append ("%Tpadding: 3px 0 3px 10px;%N")
			Result.append ("}%N%N")

			Result.append ("h1 {%N")
			Result.append ("%Tmargin-top: 5px;%N")
			Result.append ("%Tfont-size: 200%%;%N")
			Result.append ("}%N")
			Result.append ("h2 {%N")
			Result.append ("%Tfont-size: 160%%;%N")
			Result.append ("}%N")
			Result.append ("h3 {%N")
			Result.append ("%Tfont-size: 140%%;%N")
			Result.append ("}%N")
			Result.append ("h4 {%N")
			Result.append ("%Tfont-size: 120%%;%N")
			Result.append ("}%N")
			Result.append ("h5 {%N")
			Result.append ("%Tfont-size: 110%%;%N")
			Result.append ("}%N")
			Result.append ("h6 {%N")
			Result.append ("%Tfont-size: 100%%;%N")
			Result.append ("}%N%N")

			Result.append ("ol.table-of-contents, ol.table-of-contents ol {%N")
			Result.append ("%Tlist-style-type: none;%N")
			Result.append ("%Tpadding-left: 20px;%N")
			Result.append ("}%N%N")

			Result.append ("ol.table-of-contents p {%N")
			Result.append ("%Tmargin: 5px;%N")
			Result.append ("}%N%N")

			Result.append ("table {%N")
			Result.append ("border-collapse: collapse;%N")
			Result.append ("}%N%N")

			Result.append ("td, th {%N")
			Result.append ("%Tpadding: 2px 5px;%N")
			Result.append ("}%N%N")

			Result.append ("table.borders-outside-horizontal {%N")
			Result.append ("%Tborder-top: 1px solid #666;%N")
			Result.append ("%Tborder-bottom: 1px solid #666;%N")
			Result.append ("}%N")
			Result.append ("table.borders-outside-vertical {%N")
			Result.append ("%Tborder-left: 1px solid #666;%N")
			Result.append ("%Tborder-right: 1px solid #666;%N")
			Result.append ("}%N%N")

			Result.append ("table.borders-inside-horizontal td, table.borders-inside-horizontal th {%N")
			Result.append ("%Tborder-top: 1px solid #666;%N")
			Result.append ("}%N")
			Result.append ("table.borders-inside-vertical td, table.borders-inside-vertical th {%N")
			Result.append ("%Tborder-left: 1px solid #666;%N")
			Result.append ("}%N%N")

			Result.append ("table td:first-child, table th:first-child {%N")
			Result.append ("%Tborder-left: none;%N")
			Result.append ("}%N")
			Result.append ("table tr:first-child td, table tr:first-child th {%N")
			Result.append ("%Tborder-top: none;%N")
			Result.append ("}%N")

			Result.append (".syntaxhighlighter .line .content {%N")
			Result.append ("%Tborder-left: 1px solid #ccc !important;%N")
			Result.append ("}%N%N")

			Result.append (".syntaxhighlighter .line .number {%N")
			Result.append ("%Tbackground-color: #F2F2F2 !important;%N")
			Result.append ("%Tcolor: #999 !important;%N")
			Result.append ("}%N%N")

			Result.append (".syntaxhighlighter .line.alt2 {%N")
			Result.append ("%Tbackground-color: #fff !important;%N")
			Result.append ("}%N%N")

			Result.append (".syntaxhighlighter {%N")
			Result.append ("%Tpadding: 0px !important;%N")
			Result.append ("%Tborder: 1px solid #ccc !important;%N")
			Result.append ("}%N%N")

			Result.append (".syntaxhighlighter .preprocessor {%N")
			Result.append ("%Tcolor: #555 !important;%N")
			Result.append ("}%N%N")

			Result.append (".syntaxhighlighter .color1,%N")
			Result.append (".syntaxhighlighter .color1 a%N")
			Result.append ("{ %N")
			Result.append ("%Tcolor: #700000 !important; %N")
			Result.append ("}%N%N")

			Result.append (".syntaxhighlighter .functions,%N")
			Result.append (".syntaxhighlighter .color2,%N")
			Result.append (".syntaxhighlighter .color2 a {%N")
			Result.append ("%Tcolor: #000 !important;%N")
		end

invariant

	css_file_exists: css_file.exists
	css_file_is_readable: css_file.is_readable
	has_css_file_extension: attached css_file.name as name and then name.ends_with (css_file_extension)

end
